/*
 * JanelaGestao.java
 *
 * Created on 8 de Novembro de 2007, 11:10
 */
package requirementsmanager.gui;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.GregorianCalendar;
import java.util.LinkedList;
import java.util.regex.Pattern;
import javax.swing.DefaultListModel;
import javax.swing.JOptionPane;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import requirementsmanager.model.ComparacaoUtilizadorNomeInfimo;
import requirementsmanager.model.Gravavel;
import requirementsmanager.model.listeners.GravavelListener;
import requirementsmanager.model.Sistema;
import requirementsmanager.model.Utilizador;
import requirementsmanager.utils.Iterador;
import requirementsmanager.utils.ListaSimplesCircularBaseLimite;

/**
 *
 * @author  ei10608
 */
public class JanelaAdministracao extends javax.swing.JFrame implements GravavelListener {

    private Sistema sistema;
    private Pattern pattern = Pattern.compile("[\\w._%+-]+@[\\w.-]+\\.[\\w]{2,3}");
    private int numUtilizadores = 0;
    private int selectedIndex = 0;
    private DefaultListModel model;
    /**/
    private boolean byUser = true;
    private boolean emailInvalido = false;
    private boolean icqInvalido = false;
    private boolean msnInvalido = false;
    private String nomeVelho;
    private JanelaAutenticacao autenticacao;

    /** Creates new form JanelaGestao */
    public JanelaAdministracao(Sistema sistema, JanelaAutenticacao autenticacao) {
        this.autenticacao = autenticacao;
        this.sistema = sistema;
        model = new DefaultListModel();
        initModel();
        initComponents();
        jlstListaUtilizadores.setSelectedIndex(0);
        jdcDataNascimento.getDateEditor().addPropertyChangeListener(new PropertyChangeListener() {

            public void propertyChange(PropertyChangeEvent evt) {
                if (evt.getPropertyName().equals("date")) {
                    valueChanged();
                }
            }
        });
    }

    private void valueChanged() {
        if (byUser) {
            Utilizador temp = (Utilizador) model.get(selectedIndex);

            if (jdcDataNascimento.getDate() != null) {
                if (temp.getDataNascimento() == null) {
                    GregorianCalendar g = new GregorianCalendar();
                    g.setTime(jdcDataNascimento.getDate());
                    temp.setDataNascimento(g);
                } else {
                    temp.getDataNascimento().setTime(jdcDataNascimento.getDate());
                }
            } else {
                temp.setDataNascimento(null);
            }

            temp.setEndereco(jtaEndereco.getText().trim());
            temp.setCodigoPostal(sistema.getCodigoPostal(jffCodigoPostal.getText().trim(), jtfLocalidade.getText().trim()));
            temp.setCodigoExtra(jffCodigoExtra.getText().trim());
            temp.setPassword(String.valueOf(jpfPassword.getPassword()));
            temp.setEmail(jtfEmail.getText().trim());
            temp.setIcq(jtfICQ.getText().trim());
            temp.setMsn(jtfMSN.getText().trim());
            temp.setNome(jtfNome.getText().trim());
            temp.setTelefone(jffTelefone.getText().trim());
            temp.setTelemovel(jffTelemovel.getText().trim());
            temp.setUsername(jtfUsername.getText().trim());
        }
    }

    private void reordenarLista() {
        int oldIndex = 0, userId = ((Utilizador) model.get(selectedIndex)).getId();
        Utilizador u;

        ListaSimplesCircularBaseLimite lista = new ListaSimplesCircularBaseLimite(new ComparacaoUtilizadorNomeInfimo());

        for (int i = 0; i < model.getSize(); i++) {
            lista.inserir(model.get(i));
        }

        model.clear();

        Iterador it = lista.iteradorLista();
        while (it.podeAvancar()) {
            u = (Utilizador) it.avancar();
            model.addElement(u);
            if (u.getId() == userId) {
                selectedIndex = model.getSize() - 1;
            }
        }

        jlstListaUtilizadores.setSelectedIndex(oldIndex);
    }

    private void initModel() {
        LinkedList<Utilizador> lista = sistema.getUtilizadores();
        for (Utilizador u : lista) {
            u.addListener(this);
            model.addElement(u);
        }
    }

    private int encontrarIndiceOrdenado(String nome) {
        int capacity = model.size();
        for (int i = 0; i < capacity; i++) {
            if (((Utilizador) model.getElementAt(i)).getNome().compareTo(nome) >= 0) {
                return i;
            }
        }

        return -1;
    }

    private void sair() {
        if (JOptionPane.showConfirmDialog(this, java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("Tem_a_certeza_que_deseja_sair?"), java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("Deseja_Sair"), JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
            dispose();
            System.exit(0);
        }
    }

    private void novoUtilizador() {
        byUser = false;
        Utilizador temp = new Utilizador(java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("Utilizador") + (++numUtilizadores), Utilizador.INDEFINIDO);
        sistema.adicionarUtilizador(temp);

        int indice = encontrarIndiceOrdenado(temp.getNome());
        if (indice != -1) {
            model.add(indice, temp);
            jlstListaUtilizadores.setSelectedIndex(indice);
        } else {
            model.addElement(temp);
            jlstListaUtilizadores.setSelectedIndex(model.size() - 1);
        }

        jdcDataNascimento.setDate(new GregorianCalendar().getTime());
        jrbSexoMasculino.setSelected(false);
        jrbSexoFeminino.setSelected(false);
        jrbTipoGestor.setSelected(false);
        jrbTipoAdministrador.setSelected(false);

        jtfLocalidade.setText("");
        jffCodigoPostal.setText("");
        jffCodigoExtra.setText("");
        jtaEndereco.setText("");
        jpfPassword.setText("");
        jtfEmail.setText("");
        jtfICQ.setText("");
        jtfMSN.setText("");
        jtfNome.setText(temp.getNome());
        jffTelefone.setText("");
        jffTelemovel.setText("");
        jtfUsername.setText("");

        jbtnGravar.setEnabled(true);
        jmiGravar.setEnabled(true);
        jmiGravarPopUp.setEnabled(true);
        byUser = true;
    }

    private void removerUtilizador() {
        int toRemove;
        if (selectedIndex > 0) {
            toRemove = selectedIndex;
            jlstListaUtilizadores.setSelectedIndex(selectedIndex - 1);
        } else {
            toRemove = 0;
            jlstListaUtilizadores.clearSelection();
        }
        sistema.removerUtilizador((Utilizador) model.get(toRemove));
        model.remove(toRemove);
    }

    private void reiniciarPassword() {
        String pass = sistema.gerarPalavraPasse(7);
        ((Utilizador) model.get(selectedIndex)).setPassword(pass);
    }

    private void gravar() {
        if (emailInvalido) {
            JanelaErro.mostrarErro(java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("email_inváliod"), java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("Titulo"));
        } else if (msnInvalido) {
            JanelaErro.mostrarErro(java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("msn_invalido"), java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("Titulo"));
        } else if (icqInvalido) {
            JanelaErro.mostrarErro(java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("icq_invalido"), java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("titulo"));
        } else {
            sistema.gravar();
            jbtnGravar.setEnabled(false);
            jmiGravar.setEnabled(false);
            jmiGravarPopUp.setEnabled(false);
        }
        reordenarLista();
    }

    private boolean validarEmail(String email) {
        return pattern.matcher(email).matches();
    }

    private void mudarSexo() {
        if (jrbSexoFeminino.isSelected()) {
            ((Utilizador) model.get(selectedIndex)).setSexo(Utilizador.SEXO_FEMININO);
        } else {
            ((Utilizador) model.get(selectedIndex)).setSexo(Utilizador.SEXO_MASCULINO);
        }
    }

    private void mudarTipo() {
        if (jrbTipoAdministrador.isSelected()) {
            ((Utilizador) model.get(selectedIndex)).setTipo(Utilizador.ADMINISTRADOR);
        } else {
            ((Utilizador) model.get(selectedIndex)).setTipo(Utilizador.GESTOR);
        }
    }

    public void dataChanged(Gravavel g) {
        jbtnGravar.setEnabled(true);
        jmiGravar.setEnabled(true);
        jmiGravarPopUp.setEnabled(true);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        btgTipoUtilizador = new javax.swing.ButtonGroup();
        btgSexo = new javax.swing.ButtonGroup();
        jpumMenu = new javax.swing.JPopupMenu();
        jmiAdicionarUtilizadorPopUp = new javax.swing.JMenuItem();
        jmiApagarUtilizadorPopUp = new javax.swing.JMenuItem();
        jSeparator5 = new javax.swing.JSeparator();
        jmiGerarPasswordPopUp = new javax.swing.JMenuItem();
        jSeparator6 = new javax.swing.JSeparator();
        jmiGravarPopUp = new javax.swing.JMenuItem();
        jspSplit = new javax.swing.JSplitPane();
        jpDadosUtilizadores = new javax.swing.JPanel();
        jpDadosAutenticacao = new javax.swing.JPanel();
        jlblUsername = new javax.swing.JLabel();
        jtfUsername = new javax.swing.JTextField();
        jtfUsername.getDocument().addDocumentListener(new AlteracaoDocumentoListener());
        jpblPassword = new javax.swing.JLabel();
        jpfPassword = new javax.swing.JPasswordField();
        jpfPassword.getDocument().addDocumentListener(new AlteracaoDocumentoListener());
        jrbTipoAdministrador = new javax.swing.JRadioButton();
        jrbTipoGestor = new javax.swing.JRadioButton();
        jLabel14 = new javax.swing.JLabel();
        jpDadosPessoais = new javax.swing.JPanel();
        jlblNome = new javax.swing.JLabel();
        jlblDataNascimento = new javax.swing.JLabel();
        jlblSexo = new javax.swing.JLabel();
        jtfNome = new javax.swing.JTextField();
        jtfNome.getDocument().addDocumentListener(new AlteracaoDocumentoListener());
        jrbSexoMasculino = new javax.swing.JRadioButton();
        jrbSexoFeminino = new javax.swing.JRadioButton();
        jdcDataNascimento = new com.toedter.calendar.JDateChooser();
        jpContactos = new javax.swing.JPanel();
        jlblEndereco = new javax.swing.JLabel();
        jlblLocalidade = new javax.swing.JLabel();
        jlblCodigoPostal = new javax.swing.JLabel();
        jlblTelefone = new javax.swing.JLabel();
        jlblTelemovel = new javax.swing.JLabel();
        jlblEmail = new javax.swing.JLabel();
        jlblMSN = new javax.swing.JLabel();
        jlblICQ = new javax.swing.JLabel();
        jtfLocalidade = new javax.swing.JTextField();
        jtfLocalidade.getDocument().addDocumentListener(new AlteracaoDocumentoListener());
        jScrollPane2 = new javax.swing.JScrollPane();
        jtaEndereco = new javax.swing.JTextArea();
        jtaEndereco.getDocument().addDocumentListener(new AlteracaoDocumentoListener());
        jffCodigoPostal = new javax.swing.JFormattedTextField();
        jffCodigoPostal.getDocument().addDocumentListener(new AlteracaoDocumentoListener());
        jffCodigoExtra = new javax.swing.JFormattedTextField();
        jffCodigoExtra.getDocument().addDocumentListener(new AlteracaoDocumentoListener());
        jtfEmail = new javax.swing.JTextField();
        jtfEmail.getDocument().addDocumentListener(new AlteracaoDocumentoListener());
        jtfMSN = new javax.swing.JTextField();
        jtfMSN.getDocument().addDocumentListener(new AlteracaoDocumentoListener());
        jtfICQ = new javax.swing.JTextField();
        jffTelefone = new javax.swing.JFormattedTextField();
        jffTelefone.getDocument().addDocumentListener(new AlteracaoDocumentoListener());
        jffTelemovel = new javax.swing.JFormattedTextField();
        jffTelemovel.getDocument().addDocumentListener(new AlteracaoDocumentoListener());
        jScrollPane1 = new javax.swing.JScrollPane();
        jlstListaUtilizadores = new javax.swing.JList();
        jlstListaUtilizadores.setModel(model);
        jlstListaUtilizadores.addMouseListener(new JanelaAdministracao.PopupMouseListener());
        jtbBarraFerramentas = new javax.swing.JToolBar();
        jbtnAdicionarUtilizador = new javax.swing.JButton();
        jbtnRemoverUtilizador = new javax.swing.JButton();
        jsToolbarSeparator = new javax.swing.JToolBar.Separator();
        jbtnReiniciarPalavraPasse = new javax.swing.JButton();
        jSeparator4 = new javax.swing.JToolBar.Separator();
        jbtnGravar = new javax.swing.JButton();
        jmbMenu = new javax.swing.JMenuBar();
        jmFicheiro = new javax.swing.JMenu();
        jmiNovoUtilizador = new javax.swing.JMenuItem();
        jmiRemoverUtilizador = new javax.swing.JMenuItem();
        jmiReiniciarPalavraPasse = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        jmiGravar = new javax.swing.JMenuItem();
        jSeparator3 = new javax.swing.JSeparator();
        jmiLogout = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JSeparator();
        jmiSair = new javax.swing.JMenuItem();
        jmAjuda = new javax.swing.JMenu();
        jmiSobre = new javax.swing.JMenuItem();

        jmiAdicionarUtilizadorPopUp.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x16/add_user.png"))); // NOI18N
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language"); // NOI18N
        jmiAdicionarUtilizadorPopUp.setText(bundle.getString("Adicionar_Utilizador")); // NOI18N
        jmiAdicionarUtilizadorPopUp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiAdicionarUtilizadorPopUpActionPerformed(evt);
            }
        });
        jpumMenu.add(jmiAdicionarUtilizadorPopUp);

        jmiApagarUtilizadorPopUp.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x16/delete_user.png"))); // NOI18N
        jmiApagarUtilizadorPopUp.setText(bundle.getString("Remover_Utilizador")); // NOI18N
        jmiApagarUtilizadorPopUp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiApagarUtilizadorPopUpActionPerformed(evt);
            }
        });
        jpumMenu.add(jmiApagarUtilizadorPopUp);
        jpumMenu.add(jSeparator5);

        jmiGerarPasswordPopUp.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x16/restart.png"))); // NOI18N
        jmiGerarPasswordPopUp.setText(bundle.getString("Reiniciar_palavra-passe")); // NOI18N
        jmiGerarPasswordPopUp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiGerarPasswordPopUpActionPerformed(evt);
            }
        });
        jpumMenu.add(jmiGerarPasswordPopUp);
        jpumMenu.add(jSeparator6);

        jmiGravarPopUp.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x16/filesave.png"))); // NOI18N
        jmiGravarPopUp.setText(bundle.getString("Gravar")); // NOI18N
        jmiGravarPopUp.setEnabled(false);
        jmiGravarPopUp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiGravarPopUpActionPerformed(evt);
            }
        });
        jpumMenu.add(jmiGravarPopUp);

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("Administração de Utilizadores");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        jspSplit.setDividerLocation(200);

        jpDadosAutenticacao.setBorder(javax.swing.BorderFactory.createTitledBorder(" Dados de Autenticação"));

        jlblUsername.setText(bundle.getString("Nome_de_Utilizador:")); // NOI18N

        jpblPassword.setText(bundle.getString("Palavra-passe:")); // NOI18N

        jpfPassword.setText(bundle.getString("dummy")); // NOI18N

        btgTipoUtilizador.add(jrbTipoAdministrador);
        jrbTipoAdministrador.setText(bundle.getString("Administrador")); // NOI18N
        jrbTipoAdministrador.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jrbTipoAdministrador.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jrbTipoAdministrador.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jrbTipoAdministradorMouseClicked(evt);
            }
        });

        btgTipoUtilizador.add(jrbTipoGestor);
        jrbTipoGestor.setText(bundle.getString("Gestor")); // NOI18N
        jrbTipoGestor.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jrbTipoGestor.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jrbTipoGestor.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jrbTipoGestorMouseClicked(evt);
            }
        });

        jLabel14.setText(bundle.getString("Tipo_de_Utilizador:")); // NOI18N

        javax.swing.GroupLayout jpDadosAutenticacaoLayout = new javax.swing.GroupLayout(jpDadosAutenticacao);
        jpDadosAutenticacao.setLayout(jpDadosAutenticacaoLayout);
        jpDadosAutenticacaoLayout.setHorizontalGroup(
            jpDadosAutenticacaoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jpDadosAutenticacaoLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jpDadosAutenticacaoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jlblUsername)
                    .addComponent(jpblPassword)
                    .addComponent(jLabel14))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jpDadosAutenticacaoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jpDadosAutenticacaoLayout.createSequentialGroup()
                        .addComponent(jrbTipoAdministrador)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jrbTipoGestor))
                    .addGroup(jpDadosAutenticacaoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addComponent(jpfPassword, javax.swing.GroupLayout.DEFAULT_SIZE, 138, Short.MAX_VALUE)
                        .addComponent(jtfUsername, javax.swing.GroupLayout.DEFAULT_SIZE, 138, Short.MAX_VALUE)))
                .addGap(218, 218, 218))
        );
        jpDadosAutenticacaoLayout.setVerticalGroup(
            jpDadosAutenticacaoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jpDadosAutenticacaoLayout.createSequentialGroup()
                .addGroup(jpDadosAutenticacaoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jlblUsername)
                    .addComponent(jtfUsername, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(8, 8, 8)
                .addGroup(jpDadosAutenticacaoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jpblPassword)
                    .addComponent(jpfPassword, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 12, Short.MAX_VALUE)
                .addGroup(jpDadosAutenticacaoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jrbTipoAdministrador)
                    .addComponent(jrbTipoGestor)
                    .addComponent(jLabel14))
                .addContainerGap())
        );

        jpDadosPessoais.setBorder(javax.swing.BorderFactory.createTitledBorder(" Dados Pessoais "));

        jlblNome.setText(bundle.getString("Nome:")); // NOI18N

        jlblDataNascimento.setText(bundle.getString("Data_de_Nascimento:")); // NOI18N

        jlblSexo.setText(bundle.getString("Sexo:")); // NOI18N

        jtfNome.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                jtfNomeFocusGained(evt);
            }
            public void focusLost(java.awt.event.FocusEvent evt) {
                jtfNomeFocusLost(evt);
            }
        });

        btgSexo.add(jrbSexoMasculino);
        jrbSexoMasculino.setText(bundle.getString("M")); // NOI18N
        jrbSexoMasculino.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jrbSexoMasculinoMouseClicked(evt);
            }
        });

        btgSexo.add(jrbSexoFeminino);
        jrbSexoFeminino.setText(bundle.getString("F")); // NOI18N
        jrbSexoFeminino.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jrbSexoFemininoMouseClicked(evt);
            }
        });

        javax.swing.GroupLayout jpDadosPessoaisLayout = new javax.swing.GroupLayout(jpDadosPessoais);
        jpDadosPessoais.setLayout(jpDadosPessoaisLayout);
        jpDadosPessoaisLayout.setHorizontalGroup(
            jpDadosPessoaisLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jpDadosPessoaisLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jpDadosPessoaisLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jlblNome)
                    .addComponent(jlblDataNascimento)
                    .addComponent(jlblSexo))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jpDadosPessoaisLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jpDadosPessoaisLayout.createSequentialGroup()
                        .addComponent(jrbSexoMasculino)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jrbSexoFeminino))
                    .addComponent(jtfNome, javax.swing.GroupLayout.DEFAULT_SIZE, 339, Short.MAX_VALUE)
                    .addGroup(jpDadosPessoaisLayout.createSequentialGroup()
                        .addComponent(jdcDataNascimento, javax.swing.GroupLayout.DEFAULT_SIZE, 140, Short.MAX_VALUE)
                        .addGap(199, 199, 199)))
                .addContainerGap())
        );
        jpDadosPessoaisLayout.setVerticalGroup(
            jpDadosPessoaisLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jpDadosPessoaisLayout.createSequentialGroup()
                .addGroup(jpDadosPessoaisLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jlblNome)
                    .addComponent(jtfNome, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGroup(jpDadosPessoaisLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jpDadosPessoaisLayout.createSequentialGroup()
                        .addGap(11, 11, 11)
                        .addComponent(jlblDataNascimento))
                    .addGroup(jpDadosPessoaisLayout.createSequentialGroup()
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jdcDataNascimento, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addGroup(jpDadosPessoaisLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jpDadosPessoaisLayout.createSequentialGroup()
                        .addGap(8, 8, 8)
                        .addGroup(jpDadosPessoaisLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jrbSexoMasculino)
                            .addComponent(jrbSexoFeminino)))
                    .addGroup(jpDadosPessoaisLayout.createSequentialGroup()
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(jlblSexo))))
        );

        jpContactos.setBorder(javax.swing.BorderFactory.createTitledBorder(" Contactos "));

        jlblEndereco.setText("Endereço:");

        jlblLocalidade.setText(bundle.getString("Localidade:")); // NOI18N

        jlblCodigoPostal.setText("Código Postal:");

        jlblTelefone.setText(bundle.getString("Telefone:")); // NOI18N

        jlblTelemovel.setText("Telemóvel:");

        jlblEmail.setText(bundle.getString("E-mail:")); // NOI18N

        jlblMSN.setText(bundle.getString("MSN:")); // NOI18N

        jlblICQ.setText(bundle.getString("ICQ:")); // NOI18N

        jtaEndereco.setColumns(20);
        jtaEndereco.setRows(5);
        jScrollPane2.setViewportView(jtaEndereco);

        jtfEmail.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jtfEmailFocusLost(evt);
            }
        });

        jtfMSN.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jtfMSNFocusLost(evt);
            }
        });

        jtfICQ.getDocument().addDocumentListener(new AlteracaoDocumentoListener());
        jtfICQ.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jtfICQFocusLost(evt);
            }
        });

        try {
            jffTelefone.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.MaskFormatter("############")));
        } catch (java.text.ParseException ex) {
            ex.printStackTrace();
        }

        try {
            jffTelemovel.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.MaskFormatter("############")));
        } catch (java.text.ParseException ex) {
            ex.printStackTrace();
        }

        javax.swing.GroupLayout jpContactosLayout = new javax.swing.GroupLayout(jpContactos);
        jpContactos.setLayout(jpContactosLayout);
        jpContactosLayout.setHorizontalGroup(
            jpContactosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jpContactosLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jpContactosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jlblEndereco)
                    .addComponent(jlblLocalidade)
                    .addComponent(jlblCodigoPostal)
                    .addComponent(jlblTelefone)
                    .addComponent(jlblTelemovel)
                    .addComponent(jlblEmail)
                    .addComponent(jlblMSN)
                    .addComponent(jlblICQ))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jpContactosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 370, Short.MAX_VALUE)
                    .addGroup(jpContactosLayout.createSequentialGroup()
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jpContactosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jtfLocalidade, javax.swing.GroupLayout.DEFAULT_SIZE, 256, Short.MAX_VALUE)
                            .addGroup(jpContactosLayout.createSequentialGroup()
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addGroup(jpContactosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addGroup(jpContactosLayout.createSequentialGroup()
                                        .addComponent(jffCodigoPostal, javax.swing.GroupLayout.PREFERRED_SIZE, 53, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(jffCodigoExtra, javax.swing.GroupLayout.PREFERRED_SIZE, 30, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addGap(20, 20, 20))
                                    .addComponent(jtfEmail, javax.swing.GroupLayout.DEFAULT_SIZE, 256, Short.MAX_VALUE)
                                    .addComponent(jtfMSN, javax.swing.GroupLayout.DEFAULT_SIZE, 256, Short.MAX_VALUE)
                                    .addComponent(jtfICQ, javax.swing.GroupLayout.DEFAULT_SIZE, 256, Short.MAX_VALUE)
                                    .addComponent(jffTelefone, javax.swing.GroupLayout.DEFAULT_SIZE, 256, Short.MAX_VALUE)
                                    .addComponent(jffTelemovel, javax.swing.GroupLayout.DEFAULT_SIZE, 256, Short.MAX_VALUE))))
                        .addGap(114, 114, 114)))
                .addContainerGap())
        );
        jpContactosLayout.setVerticalGroup(
            jpContactosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jpContactosLayout.createSequentialGroup()
                .addGroup(jpContactosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jlblEndereco)
                    .addGroup(jpContactosLayout.createSequentialGroup()
                        .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 81, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jpContactosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jtfLocalidade, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jlblLocalidade))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jpContactosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jlblCodigoPostal)
                            .addComponent(jffCodigoPostal, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jffCodigoExtra, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jpContactosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jlblTelefone)
                    .addComponent(jffTelefone, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jpContactosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jlblTelemovel)
                    .addComponent(jffTelemovel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jpContactosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jlblEmail)
                    .addComponent(jtfEmail, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jpContactosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jlblMSN)
                    .addComponent(jtfMSN, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jpContactosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jlblICQ)
                    .addComponent(jtfICQ, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        javax.swing.GroupLayout jpDadosUtilizadoresLayout = new javax.swing.GroupLayout(jpDadosUtilizadores);
        jpDadosUtilizadores.setLayout(jpDadosUtilizadoresLayout);
        jpDadosUtilizadoresLayout.setHorizontalGroup(
            jpDadosUtilizadoresLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jpDadosUtilizadoresLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jpDadosUtilizadoresLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jpContactos, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jpDadosPessoais, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jpDadosAutenticacao, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        jpDadosUtilizadoresLayout.setVerticalGroup(
            jpDadosUtilizadoresLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jpDadosUtilizadoresLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jpDadosAutenticacao, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jpDadosPessoais, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jpContactos, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );

        jspSplit.setRightComponent(jpDadosUtilizadores);

        jlstListaUtilizadores.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                jlstListaUtilizadoresValueChanged(evt);
            }
        });
        jScrollPane1.setViewportView(jlstListaUtilizadores);

        jspSplit.setLeftComponent(jScrollPane1);

        jtbBarraFerramentas.setFloatable(false);

        jbtnAdicionarUtilizador.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x22/add_user.png"))); // NOI18N
        jbtnAdicionarUtilizador.setToolTipText(bundle.getString("Adicionar_utilizador")); // NOI18N
        jbtnAdicionarUtilizador.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbtnAdicionarUtilizadorActionPerformed(evt);
            }
        });
        jtbBarraFerramentas.add(jbtnAdicionarUtilizador);

        jbtnRemoverUtilizador.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x22/delete_user.png"))); // NOI18N
        jbtnRemoverUtilizador.setToolTipText(bundle.getString("Remover_utilizador")); // NOI18N
        jbtnRemoverUtilizador.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbtnRemoverUtilizadorActionPerformed(evt);
            }
        });
        jtbBarraFerramentas.add(jbtnRemoverUtilizador);
        jtbBarraFerramentas.add(jsToolbarSeparator);

        jbtnReiniciarPalavraPasse.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x22/restart.png"))); // NOI18N
        jbtnReiniciarPalavraPasse.setToolTipText(bundle.getString("Reiniciar_Palavra-passe")); // NOI18N
        jbtnReiniciarPalavraPasse.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbtnReiniciarPalavraPasseActionPerformed(evt);
            }
        });
        jtbBarraFerramentas.add(jbtnReiniciarPalavraPasse);
        jtbBarraFerramentas.add(jSeparator4);

        jbtnGravar.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x22/filesave.png"))); // NOI18N
        jbtnGravar.setEnabled(false);
        jbtnGravar.setFocusable(false);
        jbtnGravar.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jbtnGravar.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jbtnGravar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbtnGravarActionPerformed(evt);
            }
        });
        jtbBarraFerramentas.add(jbtnGravar);

        jmFicheiro.setText(bundle.getString("Ficheiro")); // NOI18N

        jmiNovoUtilizador.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x16/add_user.png"))); // NOI18N
        jmiNovoUtilizador.setText(bundle.getString("Novo_Utilizador")); // NOI18N
        jmiNovoUtilizador.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiNovoUtilizadorActionPerformed(evt);
            }
        });
        jmFicheiro.add(jmiNovoUtilizador);

        jmiRemoverUtilizador.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x16/delete_user.png"))); // NOI18N
        jmiRemoverUtilizador.setText(bundle.getString("Remover_Utilizador")); // NOI18N
        jmiRemoverUtilizador.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiRemoverUtilizadorActionPerformed(evt);
            }
        });
        jmFicheiro.add(jmiRemoverUtilizador);

        jmiReiniciarPalavraPasse.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x16/restart.png"))); // NOI18N
        jmiReiniciarPalavraPasse.setText(bundle.getString("Reiniciar_Palavra-passe")); // NOI18N
        jmiReiniciarPalavraPasse.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiReiniciarPalavraPasseActionPerformed(evt);
            }
        });
        jmFicheiro.add(jmiReiniciarPalavraPasse);
        jmFicheiro.add(jSeparator1);

        jmiGravar.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x16/filesave.png"))); // NOI18N
        jmiGravar.setText(bundle.getString("Gravar")); // NOI18N
        jmiGravar.setEnabled(false);
        jmiGravar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiGravarActionPerformed(evt);
            }
        });
        jmFicheiro.add(jmiGravar);
        jmFicheiro.add(jSeparator3);

        jmiLogout.setText(bundle.getString("Logout")); // NOI18N
        jmiLogout.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiLogoutActionPerformed(evt);
            }
        });
        jmFicheiro.add(jmiLogout);
        jmFicheiro.add(jSeparator2);

        jmiSair.setText(bundle.getString("Sair")); // NOI18N
        jmiSair.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiSairActionPerformed(evt);
            }
        });
        jmFicheiro.add(jmiSair);

        jmbMenu.add(jmFicheiro);

        jmAjuda.setText(bundle.getString("Ajuda")); // NOI18N

        jmiSobre.setText(bundle.getString("Sobre")); // NOI18N
        jmiSobre.setEnabled(false);
        jmiSobre.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiSobreActionPerformed(evt);
            }
        });
        jmAjuda.add(jmiSobre);

        jmbMenu.add(jmAjuda);

        setJMenuBar(jmbMenu);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jspSplit, javax.swing.GroupLayout.DEFAULT_SIZE, 705, Short.MAX_VALUE)
                .addContainerGap())
            .addComponent(jtbBarraFerramentas, javax.swing.GroupLayout.DEFAULT_SIZE, 725, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jtbBarraFerramentas, javax.swing.GroupLayout.PREFERRED_SIZE, 25, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jspSplit, javax.swing.GroupLayout.DEFAULT_SIZE, 563, Short.MAX_VALUE)
                .addContainerGap())
        );

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-733)/2, (screenSize.height-660)/2, 733, 660);
    }// </editor-fold>//GEN-END:initComponents
    private void jmiSairActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiSairActionPerformed
        sair();
    }//GEN-LAST:event_jmiSairActionPerformed

    private void jmiSobreActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiSobreActionPerformed
        throw new UnsupportedOperationException(java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("Por_implementar"));
    }//GEN-LAST:event_jmiSobreActionPerformed

    private void jmiNovoUtilizadorActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiNovoUtilizadorActionPerformed
        novoUtilizador();
    }//GEN-LAST:event_jmiNovoUtilizadorActionPerformed

    private void jmiRemoverUtilizadorActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiRemoverUtilizadorActionPerformed
        removerUtilizador();
    }//GEN-LAST:event_jmiRemoverUtilizadorActionPerformed

    private void jmiReiniciarPalavraPasseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiReiniciarPalavraPasseActionPerformed
        reiniciarPassword();
    }//GEN-LAST:event_jmiReiniciarPalavraPasseActionPerformed

    private void jbtnReiniciarPalavraPasseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnReiniciarPalavraPasseActionPerformed
        reiniciarPassword();
    }//GEN-LAST:event_jbtnReiniciarPalavraPasseActionPerformed

    private void jbtnRemoverUtilizadorActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnRemoverUtilizadorActionPerformed
        removerUtilizador();
    }//GEN-LAST:event_jbtnRemoverUtilizadorActionPerformed

    private void jbtnAdicionarUtilizadorActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnAdicionarUtilizadorActionPerformed
        novoUtilizador();
    }//GEN-LAST:event_jbtnAdicionarUtilizadorActionPerformed

    private void jmiGravarActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiGravarActionPerformed
        gravar();
    }//GEN-LAST:event_jmiGravarActionPerformed

    private void jlstListaUtilizadoresValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_jlstListaUtilizadoresValueChanged
        selectedIndex = jlstListaUtilizadores.getSelectedIndex();
        if (selectedIndex >= 0) {
            byUser = false;
            Utilizador actual = (Utilizador) model.get(selectedIndex);
            if (actual != null) {
                if (actual.getDataNascimento() != null) {
                    jdcDataNascimento.setDate(actual.getDataNascimento().getTime());
                } else {
                    jdcDataNascimento.setDate(null);
                }

                if (actual.getSexo().equals(java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("M"))) {
                    jrbSexoMasculino.setSelected(true);
                } else {
                    jrbSexoFeminino.setSelected(true);
                }

                switch (actual.getTipo()) {
                    case Utilizador.ADMINISTRADOR:
                        jrbTipoAdministrador.setSelected(true);
                        break;
                    case Utilizador.GESTOR:
                        jrbTipoGestor.setSelected(true);
                }

                if (actual.getCodigoPostal() != null) {
                    jtfLocalidade.setText(actual.getCodigoPostal().getLocalidade());
                    jffCodigoPostal.setText(actual.getCodigoPostal().getCodigo());
                } else {
                    jtfLocalidade.setText("");
                    jffCodigoPostal.setText("");
                }

                jffCodigoExtra.setText(actual.getCodigoExtra());
                jtaEndereco.setText(actual.getEndereco());
                jpfPassword.setText(java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("dummy"));
                jtfEmail.setText(actual.getEmail());
                jtfICQ.setText(actual.getIcq());
                jtfMSN.setText(actual.getMsn());
                jtfNome.setText(actual.getNome());
                jffTelefone.setText(actual.getTelemovel());
                jffTelemovel.setText(actual.getTelemovel());
                jtfUsername.setText(actual.getUsername());
            }
            byUser = true;
        }
    }//GEN-LAST:event_jlstListaUtilizadoresValueChanged

    private void jmiLogoutActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiLogoutActionPerformed
        sistema.logout();
        dispose();
        autenticacao.setVisible(true);
    }//GEN-LAST:event_jmiLogoutActionPerformed

    private void jbtnGravarActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnGravarActionPerformed
        gravar();
    }//GEN-LAST:event_jbtnGravarActionPerformed

    private void jrbSexoFemininoMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jrbSexoFemininoMouseClicked
        mudarSexo();
    }//GEN-LAST:event_jrbSexoFemininoMouseClicked

    private void jrbSexoMasculinoMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jrbSexoMasculinoMouseClicked
        mudarSexo();
    }//GEN-LAST:event_jrbSexoMasculinoMouseClicked

    private void jrbTipoAdministradorMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jrbTipoAdministradorMouseClicked
        mudarTipo();
    }//GEN-LAST:event_jrbTipoAdministradorMouseClicked

    private void jrbTipoGestorMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jrbTipoGestorMouseClicked
        mudarTipo();
    }//GEN-LAST:event_jrbTipoGestorMouseClicked

    private void jtfEmailFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jtfEmailFocusLost
        if (!validarEmail(jtfEmail.getText())) {
            emailInvalido = true;
        }
    }//GEN-LAST:event_jtfEmailFocusLost

    private void jtfMSNFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jtfMSNFocusLost
        if (!validarEmail(jtfMSN.getText().trim())) {
            msnInvalido = true;
        }
    }//GEN-LAST:event_jtfMSNFocusLost

    private void jtfICQFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jtfICQFocusLost
        if (!validarEmail(jtfICQ.getText().trim())) {
            icqInvalido = true;
        }
    }//GEN-LAST:event_jtfICQFocusLost

    private void jtfNomeFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jtfNomeFocusGained
        nomeVelho = jtfUsername.getText().trim();
    }//GEN-LAST:event_jtfNomeFocusGained

    private void jtfNomeFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jtfNomeFocusLost
        if (!nomeVelho.equals(jtfUsername.getText().trim())) {
            reordenarLista();
        }
    }//GEN-LAST:event_jtfNomeFocusLost

    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        sair();
    }//GEN-LAST:event_formWindowClosing

    private void jmiAdicionarUtilizadorPopUpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiAdicionarUtilizadorPopUpActionPerformed
        novoUtilizador();
    }//GEN-LAST:event_jmiAdicionarUtilizadorPopUpActionPerformed

    private void jmiApagarUtilizadorPopUpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiApagarUtilizadorPopUpActionPerformed
        removerUtilizador();
    }//GEN-LAST:event_jmiApagarUtilizadorPopUpActionPerformed

    private void jmiGerarPasswordPopUpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiGerarPasswordPopUpActionPerformed
        reiniciarPassword();
    }//GEN-LAST:event_jmiGerarPasswordPopUpActionPerformed

    private void jmiGravarPopUpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiGravarPopUpActionPerformed
        gravar();
    }//GEN-LAST:event_jmiGravarPopUpActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup btgSexo;
    private javax.swing.ButtonGroup btgTipoUtilizador;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JSeparator jSeparator3;
    private javax.swing.JToolBar.Separator jSeparator4;
    private javax.swing.JSeparator jSeparator5;
    private javax.swing.JSeparator jSeparator6;
    private javax.swing.JButton jbtnAdicionarUtilizador;
    private javax.swing.JButton jbtnGravar;
    private javax.swing.JButton jbtnReiniciarPalavraPasse;
    private javax.swing.JButton jbtnRemoverUtilizador;
    private com.toedter.calendar.JDateChooser jdcDataNascimento;
    private javax.swing.JFormattedTextField jffCodigoExtra;
    private javax.swing.JFormattedTextField jffCodigoPostal;
    private javax.swing.JFormattedTextField jffTelefone;
    private javax.swing.JFormattedTextField jffTelemovel;
    private javax.swing.JLabel jlblCodigoPostal;
    private javax.swing.JLabel jlblDataNascimento;
    private javax.swing.JLabel jlblEmail;
    private javax.swing.JLabel jlblEndereco;
    private javax.swing.JLabel jlblICQ;
    private javax.swing.JLabel jlblLocalidade;
    private javax.swing.JLabel jlblMSN;
    private javax.swing.JLabel jlblNome;
    private javax.swing.JLabel jlblSexo;
    private javax.swing.JLabel jlblTelefone;
    private javax.swing.JLabel jlblTelemovel;
    private javax.swing.JLabel jlblUsername;
    private javax.swing.JList jlstListaUtilizadores;
    private javax.swing.JMenu jmAjuda;
    private javax.swing.JMenu jmFicheiro;
    private javax.swing.JMenuBar jmbMenu;
    private javax.swing.JMenuItem jmiAdicionarUtilizadorPopUp;
    private javax.swing.JMenuItem jmiApagarUtilizadorPopUp;
    private javax.swing.JMenuItem jmiGerarPasswordPopUp;
    private javax.swing.JMenuItem jmiGravar;
    private javax.swing.JMenuItem jmiGravarPopUp;
    private javax.swing.JMenuItem jmiLogout;
    private javax.swing.JMenuItem jmiNovoUtilizador;
    private javax.swing.JMenuItem jmiReiniciarPalavraPasse;
    private javax.swing.JMenuItem jmiRemoverUtilizador;
    private javax.swing.JMenuItem jmiSair;
    private javax.swing.JMenuItem jmiSobre;
    private javax.swing.JPanel jpContactos;
    private javax.swing.JPanel jpDadosAutenticacao;
    private javax.swing.JPanel jpDadosPessoais;
    private javax.swing.JPanel jpDadosUtilizadores;
    private javax.swing.JLabel jpblPassword;
    private javax.swing.JPasswordField jpfPassword;
    private javax.swing.JPopupMenu jpumMenu;
    private javax.swing.JRadioButton jrbSexoFeminino;
    private javax.swing.JRadioButton jrbSexoMasculino;
    private javax.swing.JRadioButton jrbTipoAdministrador;
    private javax.swing.JRadioButton jrbTipoGestor;
    private javax.swing.JToolBar.Separator jsToolbarSeparator;
    private javax.swing.JSplitPane jspSplit;
    private javax.swing.JTextArea jtaEndereco;
    private javax.swing.JToolBar jtbBarraFerramentas;
    private javax.swing.JTextField jtfEmail;
    private javax.swing.JTextField jtfICQ;
    private javax.swing.JTextField jtfLocalidade;
    private javax.swing.JTextField jtfMSN;
    private javax.swing.JTextField jtfNome;
    private javax.swing.JTextField jtfUsername;
    // End of variables declaration//GEN-END:variables
    private class AlteracaoDocumentoListener implements DocumentListener {

        public void insertUpdate(DocumentEvent e) {
            valueChanged();
        }

        public void removeUpdate(DocumentEvent e) {
            valueChanged();
        }

        public void changedUpdate(DocumentEvent e) {
        //do nothing
        }
    }

    private class PopupMouseListener extends MouseAdapter {

        @Override
        public void mousePressed(MouseEvent e) {
            tryToShowPopup(e);
        }

        @Override
        public void mouseReleased(MouseEvent e) {
            tryToShowPopup(e);
        }

        private void tryToShowPopup(MouseEvent e) {
            if (e.isPopupTrigger()) {
                jpumMenu.show(e.getComponent(), e.getX(), e.getY());
            }
        }
    }
}
