/*
 * JanelaGestao.java
 *
 * Created on 8 de Novembro de 2007, 11:10
 */
package requirementsmanager.gui;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.text.Position;
import javax.swing.JOptionPane;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreePath;
import requirementsmanager.model.CasoUso;
import requirementsmanager.model.Gestor;
import requirementsmanager.model.Gravavel;
import requirementsmanager.model.Projecto;
import requirementsmanager.model.Requisito;
import requirementsmanager.model.Sistema;
import requirementsmanager.model.Teste;
import requirementsmanager.model.listeners.GravavelListener;

/**
 *
 * @author  ei10608
 */
public class JanelaGestao extends javax.swing.JFrame implements GravavelListener {

    private static final String TREE_PROJECTO = java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("Projectos");
    private static final String TREE_CASOUSO = java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("Casos_de_Uso");
    private static final String TREE_REQUISITO = java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("Requisitos");
    private static final String TREE_TESTE = java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("Testes");
    private static final int TREE_OFFSET_CASOUSO = 0;
    private static final int TREE_OFFSET_REQUISITO = 1;
    private static final int TREE_OFFSET_TESTE = 2;
    private PainelGestaoCasosUso panelGestaoCasosUso;
    private PainelGestaoRequisitos panelGestaoRequisitos;
    private PainelGestaoTestesAssociados panelGestaoTestesAssociados;
    private PainelGestaoProjecto panelGestaoProjectos;
    private Projecto activo;
    private Sistema sist;
    private Gestor user;
    private JanelaGestao me = this;
    private JanelaAutenticacao autenticacao;
    private int numProjectos = 0;
    private int numCasosUso = 0;
    private int numRequisitos = 0;
    private int numTestes = 0;

    public JanelaGestao(Sistema sist, Gestor user, JanelaAutenticacao autenticacao) {
        this.autenticacao = autenticacao;
        this.sist = sist;
        this.user = user;
        DefaultTreeModel model;
        MutableTreeNode root, nodePai, nodeFilho, nodeNeto;

        panelGestaoCasosUso = new PainelGestaoCasosUso(sist);
        panelGestaoRequisitos = new PainelGestaoRequisitos(sist);
        panelGestaoTestesAssociados = new PainelGestaoTestesAssociados(sist);
        panelGestaoProjectos = new PainelGestaoProjecto(sist);

        initComponents();

        treeGestao.setModel(model = new DefaultTreeModel(root = new DefaultMutableTreeNode(TREE_PROJECTO)));
        if (user.getProjectosValidos().size() > 0) {//added for null
            activo = user.getProjectosValidos().getFirst();
            for (Projecto p : user.getProjectosValidos()) {
                p.addListener(this);
                nodePai = new DefaultMutableTreeNode(p);
                model.insertNodeInto(nodePai, root, root.getChildCount());

                nodeFilho = new DefaultMutableTreeNode(TREE_CASOUSO);
                model.insertNodeInto(nodeFilho, nodePai, nodePai.getChildCount());
                for (CasoUso c : p.getListaCasosUsoValidos()) {
                    c.addListener(this);
                    nodeNeto = new DefaultMutableTreeNode(c);
                    model.insertNodeInto(nodeNeto, nodeFilho, nodeFilho.getChildCount());
                }

                nodeFilho = new DefaultMutableTreeNode(TREE_REQUISITO);
                model.insertNodeInto(nodeFilho, nodePai, nodePai.getChildCount());
                for (Requisito r : p.getListaRequisitosValidos()) {
                    r.addListener(this);
                    nodeNeto = new DefaultMutableTreeNode(r);
                    model.insertNodeInto(nodeNeto, nodeFilho, nodeFilho.getChildCount());
                }

                nodeFilho = new DefaultMutableTreeNode(TREE_TESTE);
                model.insertNodeInto(nodeFilho, nodePai, nodePai.getChildCount());
                for (Teste t : p.getListaTestesValidos()) {
                    t.addListener(this);
                    nodeNeto = new DefaultMutableTreeNode(t);
                    model.insertNodeInto(nodeNeto, nodeFilho, nodeFilho.getChildCount());
                }
            }
            treeGestao.expandRow(0);
            if (activo != null) {
                jbtnGerarRelatorio.setEnabled(true);
            }
        }//added for null
    }

    private void mostrarDadosPessoais() {
        System.err.println(java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("2:_") + user.getUsername());
        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                new JanelaDadosPessoais(me, sist, user).setVisible(true);
            }
        });
    }

    private void sair() {
        if (JOptionPane.showConfirmDialog(this, java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("Tem_a_certeza_que_deseja_sair?"), java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("Deseja_Sair"), JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
            dispose();
            System.exit(0);
        }
    }

    private void gravar() {
        sist.gravar();
        jbtnGravar.setEnabled(false);
        jmiGravar.setEnabled(false);
    }

    private void novoProjecto() {
        Projecto p = new Projecto(-1, java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("Novo_Projecto_") + ++numProjectos, java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString(""), null, null, null, null);
        p.addListener(this);

        DefaultMutableTreeNode root, nodePai, nodeFilho;
        DefaultTreeModel model = (DefaultTreeModel) treeGestao.getModel();
        root = (DefaultMutableTreeNode) model.getRoot();
        model.insertNodeInto(nodePai = new DefaultMutableTreeNode(p), root, root.getChildCount());
        nodeFilho = new DefaultMutableTreeNode(TREE_CASOUSO);
        model.insertNodeInto(nodeFilho, nodePai, nodePai.getChildCount());
        nodeFilho = new DefaultMutableTreeNode(TREE_PROJECTO);
        model.insertNodeInto(nodeFilho, nodePai, nodePai.getChildCount());
        nodeFilho = new DefaultMutableTreeNode(TREE_TESTE);
        model.insertNodeInto(nodeFilho, nodePai, nodePai.getChildCount());

        TreePath path;
        treeGestao.expandPath(path = new TreePath(nodePai.getPath()));
        treeGestao.setSelectionPath(path);
        treeGestao.scrollPathToVisible(path);
        panelGestaoProjectos.mostarProjecto(p);
        sist.adicionarProjecto(p);
        jbtnGravar.setEnabled(true);
        jmiGravar.setEnabled(true);
    }

    public void addNewCasoUso() {
        if (activo != null) {
            CasoUso c = new CasoUso(-1, java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("Novo_Caso_de_Uso_") + ++numCasosUso, java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString(""), null, activo, null);
            c.addListener(this);
            activo.adicionarCasoUso(c);
            c.setProjecto(activo);
            addNodeTree(c, TREE_OFFSET_CASOUSO);
        }
    }

    public void addNewRequisito() {
        if (activo != null) {
            Requisito r = new Requisito(-1, java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("Novo_Requisito_") + ++numRequisitos, Requisito.ORIGEM_CLIENTE, Requisito.RISCO_BAIXO, Requisito.PRIORIDADE_BAIXA, java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString(""), activo);
            r.addListener(this);
            activo.adicionarRequisito(r);
            r.setProjecto(activo);
            addNodeTree(r, TREE_OFFSET_REQUISITO);
        }
    }

    public void addNewTeste() {
        if (activo != null) {
            Teste t = new Teste(-1, java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("Novo_Teste_") + ++numTestes, sist.getTiposTeste().getFirst(), false, activo, null);
            t.addListener(this);
            activo.adicionarTeste(t);
            t.setProjecto(activo);
            addNodeTree(t, TREE_OFFSET_TESTE);
        }
    }

    @SuppressWarnings("empty-statement")
    private void addNodeTree(Object o, int offset) {
        TreePath path;
        while (((path = treeGestao.getNextMatch(activo.toString(), 0, Position.Bias.Forward)) != null) && !((DefaultMutableTreeNode) path.getLastPathComponent()).getUserObject().equals(activo)) {
            ;
        }
        MutableTreeNode node = (MutableTreeNode) path.getLastPathComponent();
        node = (MutableTreeNode) node.getChildAt(offset);

        DefaultMutableTreeNode nodeFilho;
        DefaultTreeModel model = (DefaultTreeModel) treeGestao.getModel();
        model.insertNodeInto(nodeFilho = new DefaultMutableTreeNode(o), node, node.getChildCount());

        treeGestao.setSelectionPath(new TreePath(nodeFilho.getPath()));
        jbtnGravar.setEnabled(true);
        jmiGravar.setEnabled(true);

    }

    private void removerDados() {
        TreePath path;
        String name;
        DefaultMutableTreeNode node = null;
        if ((path = treeGestao.getSelectionPath()) != null) {
            if (path.getPathCount() == 2) {
                Projecto p = (Projecto) (node = (DefaultMutableTreeNode) path.getLastPathComponent()).getUserObject();
                p.removeListener(this);
                sist.remover(p);
                ((DefaultTreeModel) treeGestao.getModel()).removeNodeFromParent(node);
            } else if (path.getPathCount() == 4) {
                name = path.getPathComponent(path.getPathCount() - 2).toString();
                if (name.equals(TREE_CASOUSO)) {
                    CasoUso c = (CasoUso) (node = (DefaultMutableTreeNode) path.getLastPathComponent()).getUserObject();
                    c.removeListener(this);
                    sist.remover(c);
                } else if (name.equals(TREE_REQUISITO)) {
                    Requisito r = (Requisito) (node = (DefaultMutableTreeNode) path.getLastPathComponent()).getUserObject();
                    r.removeListener(this);
                    sist.remover(r);
                } else if (name.equals(TREE_TESTE)) {
                    Teste t = (Teste) (node = (DefaultMutableTreeNode) path.getLastPathComponent()).getUserObject();
                    t.removeListener(this);
                    sist.remover(t);
                }
                treeGestao.removeSelectionPath(path);
                ((DefaultTreeModel) treeGestao.getModel()).removeNodeFromParent(node);
            }
            jbtnGravar.setEnabled(true);
            jmiGravar.setEnabled(true);
        }
    }

    private void tornarActivo() {
        TreePath path = treeGestao.getLeadSelectionPath();
        if (path != null && path.getPathCount() == 2) {
            activo = (Projecto) ((DefaultMutableTreeNode) path.getPath()[path.getPathCount() - 1]).getUserObject();
            jbtnGerarRelatorio.setEnabled(true);
        }
    }

    public void editado() {
        jbtnGravar.setEnabled(true);
        jmiGravar.setEnabled(true);
    }

    public void dataChanged(Gravavel g) {
        jbtnGravar.setEnabled(true);
        jmiGravar.setEnabled(true);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jpumMenu = new javax.swing.JPopupMenu();
        jmiNovoProjectoPopUp = new javax.swing.JMenuItem();
        jmiNovoRequisitoPopUp = new javax.swing.JMenuItem();
        jmiNovoCasoUsoPopUp = new javax.swing.JMenuItem();
        jmiNovoTestePopUp = new javax.swing.JMenuItem();
        jSeparator10 = new javax.swing.JSeparator();
        jmiEliminarPopUp = new javax.swing.JMenuItem();
        jSeparator11 = new javax.swing.JSeparator();
        jmiMarcarActivoPopUp = new javax.swing.JMenuItem();
        jSplitPane = new javax.swing.JSplitPane();
        panelPlacer = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        treeGestao = new javax.swing.JTree();
        treeGestao.addMouseListener(new PopupMouseListener());
        jtbBarraFerramentas = new javax.swing.JToolBar();
        jbtnAdicionarProjecto = new javax.swing.JButton();
        jbtnAdicionarRequisito = new javax.swing.JButton();
        jbtnAdicionarCasoUso = new javax.swing.JButton();
        jbtnAdicionarTeste = new javax.swing.JButton();
        jSeparator3 = new javax.swing.JToolBar.Separator();
        jbtnRemover = new javax.swing.JButton();
        jSeparator4 = new javax.swing.JToolBar.Separator();
        jbtnGravar = new javax.swing.JButton();
        jSeparator5 = new javax.swing.JToolBar.Separator();
        jbtnGerarRelatorio = new javax.swing.JButton();
        jbtnVerDadosPessoais = new javax.swing.JButton();
        jmbBarraMenu = new javax.swing.JMenuBar();
        jmFicheiro = new javax.swing.JMenu();
        jmNovo = new javax.swing.JMenu();
        jmiNovoProjecto = new javax.swing.JMenuItem();
        jmiNovoRequisito = new javax.swing.JMenuItem();
        jmiNovoCasoUso = new javax.swing.JMenuItem();
        jmiNovoTeste = new javax.swing.JMenuItem();
        jmiEliminar = new javax.swing.JMenuItem();
        jSeparator7 = new javax.swing.JSeparator();
        jmiGravar = new javax.swing.JMenuItem();
        jSeparator9 = new javax.swing.JSeparator();
        jmiLogout = new javax.swing.JMenuItem();
        jSeparator8 = new javax.swing.JSeparator();
        jmiSair = new javax.swing.JMenuItem();
        jmVer = new javax.swing.JMenu();
        jmiListarProjectos = new javax.swing.JMenuItem();
        jmiListarCUSemR = new javax.swing.JMenuItem();
        jmiListarRSemCU = new javax.swing.JMenuItem();
        jSeparator6 = new javax.swing.JSeparator();
        jmiDadosUtilizadores = new javax.swing.JMenuItem();
        jmFerramentas = new javax.swing.JMenu();
        jmiGerirTiposTeste = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        jmiTornarActivo = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JSeparator();
        jmiAlterarDadosPessoais = new javax.swing.JMenuItem();
        jmAjuda = new javax.swing.JMenu();
        jmiSobre = new javax.swing.JMenuItem();

        jmiNovoProjectoPopUp.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x16/add_projecto.png"))); // NOI18N
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language"); // NOI18N
        jmiNovoProjectoPopUp.setText(bundle.getString("Novo_Projecto")); // NOI18N
        jmiNovoProjectoPopUp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiNovoProjectoPopUpActionPerformed(evt);
            }
        });
        jpumMenu.add(jmiNovoProjectoPopUp);

        jmiNovoRequisitoPopUp.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x16/add_requisito.png"))); // NOI18N
        jmiNovoRequisitoPopUp.setText(bundle.getString("Novo_Requisito")); // NOI18N
        jmiNovoRequisitoPopUp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiNovoRequisitoPopUpActionPerformed(evt);
            }
        });
        jpumMenu.add(jmiNovoRequisitoPopUp);

        jmiNovoCasoUsoPopUp.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x16/add_casoUso.png"))); // NOI18N
        jmiNovoCasoUsoPopUp.setText(bundle.getString("Novo_Caso_de_Uso")); // NOI18N
        jmiNovoCasoUsoPopUp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiNovoCasoUsoPopUpActionPerformed(evt);
            }
        });
        jpumMenu.add(jmiNovoCasoUsoPopUp);

        jmiNovoTestePopUp.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x16/add_teste.png"))); // NOI18N
        jmiNovoTestePopUp.setText(bundle.getString("Novo_Teste")); // NOI18N
        jmiNovoTestePopUp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiNovoTestePopUpActionPerformed(evt);
            }
        });
        jpumMenu.add(jmiNovoTestePopUp);
        jpumMenu.add(jSeparator10);

        jmiEliminarPopUp.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x16/edit_remove.png"))); // NOI18N
        jmiEliminarPopUp.setText(bundle.getString("Eliminar")); // NOI18N
        jmiEliminarPopUp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiEliminarPopUpActionPerformed(evt);
            }
        });
        jpumMenu.add(jmiEliminarPopUp);
        jpumMenu.add(jSeparator11);

        jmiMarcarActivoPopUp.setText(bundle.getString("Marcar_com_activo")); // NOI18N
        jmiMarcarActivoPopUp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiMarcarActivoPopUpActionPerformed(evt);
            }
        });
        jpumMenu.add(jmiMarcarActivoPopUp);

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("Requirements Manager - Janela de Gestão");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        jSplitPane.setDividerLocation(240);

        javax.swing.GroupLayout panelPlacerLayout = new javax.swing.GroupLayout(panelPlacer);
        panelPlacer.setLayout(panelPlacerLayout);
        panelPlacerLayout.setHorizontalGroup(
            panelPlacerLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 537, Short.MAX_VALUE)
        );
        panelPlacerLayout.setVerticalGroup(
            panelPlacerLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 603, Short.MAX_VALUE)
        );

        jSplitPane.setRightComponent(panelPlacer);

        treeGestao.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
                treeGestaoValueChanged(evt);
            }
        });
        jScrollPane1.setViewportView(treeGestao);

        jSplitPane.setLeftComponent(jScrollPane1);

        jtbBarraFerramentas.setFloatable(false);

        jbtnAdicionarProjecto.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x22/add_projecto.png"))); // NOI18N
        jbtnAdicionarProjecto.setFocusable(false);
        jbtnAdicionarProjecto.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jbtnAdicionarProjecto.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbtnAdicionarProjectoActionPerformed(evt);
            }
        });
        jtbBarraFerramentas.add(jbtnAdicionarProjecto);

        jbtnAdicionarRequisito.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x22/add_requisito.png"))); // NOI18N
        jbtnAdicionarRequisito.setFocusable(false);
        jbtnAdicionarRequisito.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jbtnAdicionarRequisito.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jbtnAdicionarRequisito.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbtnAdicionarRequisitoActionPerformed(evt);
            }
        });
        jtbBarraFerramentas.add(jbtnAdicionarRequisito);

        jbtnAdicionarCasoUso.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x22/add_casoUso.png"))); // NOI18N
        jbtnAdicionarCasoUso.setFocusable(false);
        jbtnAdicionarCasoUso.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jbtnAdicionarCasoUso.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbtnAdicionarCasoUsoActionPerformed(evt);
            }
        });
        jtbBarraFerramentas.add(jbtnAdicionarCasoUso);

        jbtnAdicionarTeste.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x22/add_teste.png"))); // NOI18N
        jbtnAdicionarTeste.setFocusable(false);
        jbtnAdicionarTeste.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jbtnAdicionarTeste.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jbtnAdicionarTeste.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbtnAdicionarTesteActionPerformed(evt);
            }
        });
        jtbBarraFerramentas.add(jbtnAdicionarTeste);
        jtbBarraFerramentas.add(jSeparator3);

        jbtnRemover.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x22/edit_remove.png"))); // NOI18N
        jbtnRemover.setFocusable(false);
        jbtnRemover.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jbtnRemover.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jbtnRemover.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbtnRemoverActionPerformed(evt);
            }
        });
        jtbBarraFerramentas.add(jbtnRemover);
        jtbBarraFerramentas.add(jSeparator4);

        jbtnGravar.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x22/filesave.png"))); // NOI18N
        jbtnGravar.setEnabled(false);
        jbtnGravar.setFocusable(false);
        jbtnGravar.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jbtnGravar.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jbtnGravar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbtnGravarActionPerformed(evt);
            }
        });
        jtbBarraFerramentas.add(jbtnGravar);
        jtbBarraFerramentas.add(jSeparator5);

        jbtnGerarRelatorio.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x22/pie.png"))); // NOI18N
        jbtnGerarRelatorio.setEnabled(false);
        jbtnGerarRelatorio.setFocusable(false);
        jbtnGerarRelatorio.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jbtnGerarRelatorio.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jbtnGerarRelatorio.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbtnGerarRelatorioActionPerformed(evt);
            }
        });
        jtbBarraFerramentas.add(jbtnGerarRelatorio);

        jbtnVerDadosPessoais.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x22/edit_user.png"))); // NOI18N
        jbtnVerDadosPessoais.setFocusable(false);
        jbtnVerDadosPessoais.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jbtnVerDadosPessoais.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jbtnVerDadosPessoais.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbtnVerDadosPessoaisActionPerformed(evt);
            }
        });
        jtbBarraFerramentas.add(jbtnVerDadosPessoais);

        jmFicheiro.setText(bundle.getString("Ficheiro")); // NOI18N

        jmNovo.setText(bundle.getString("Menu")); // NOI18N

        jmiNovoProjecto.setText(bundle.getString("Projecto")); // NOI18N
        jmiNovoProjecto.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiNovoProjectoActionPerformed(evt);
            }
        });
        jmNovo.add(jmiNovoProjecto);

        jmiNovoRequisito.setText(bundle.getString("Requisito")); // NOI18N
        jmiNovoRequisito.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiNovoRequisitoActionPerformed(evt);
            }
        });
        jmNovo.add(jmiNovoRequisito);

        jmiNovoCasoUso.setText(bundle.getString("Caso_de_Uso")); // NOI18N
        jmiNovoCasoUso.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiNovoCasoUsoActionPerformed(evt);
            }
        });
        jmNovo.add(jmiNovoCasoUso);

        jmiNovoTeste.setText(bundle.getString("Teste")); // NOI18N
        jmiNovoTeste.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiNovoTesteActionPerformed(evt);
            }
        });
        jmNovo.add(jmiNovoTeste);

        jmFicheiro.add(jmNovo);

        jmiEliminar.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x16/edit_remove.png"))); // NOI18N
        jmiEliminar.setText(bundle.getString("Eliminar")); // NOI18N
        jmiEliminar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiEliminarActionPerformed(evt);
            }
        });
        jmFicheiro.add(jmiEliminar);
        jmFicheiro.add(jSeparator7);

        jmiGravar.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_G, java.awt.event.InputEvent.ALT_MASK));
        jmiGravar.setIcon(new javax.swing.ImageIcon(getClass().getResource("/requirementsmanager/resources/images/x16/filesave.png"))); // NOI18N
        jmiGravar.setText(bundle.getString("Gravar")); // NOI18N
        jmiGravar.setEnabled(false);
        jmiGravar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiGravarActionPerformed(evt);
            }
        });
        jmFicheiro.add(jmiGravar);
        jmFicheiro.add(jSeparator9);

        jmiLogout.setText(bundle.getString("Logout")); // NOI18N
        jmiLogout.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiLogoutActionPerformed(evt);
            }
        });
        jmFicheiro.add(jmiLogout);
        jmFicheiro.add(jSeparator8);

        jmiSair.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F4, java.awt.event.InputEvent.ALT_MASK));
        jmiSair.setText(bundle.getString("Sair")); // NOI18N
        jmiSair.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiSairActionPerformed(evt);
            }
        });
        jmFicheiro.add(jmiSair);

        jmbBarraMenu.add(jmFicheiro);

        jmVer.setText(bundle.getString("Ver")); // NOI18N

        jmiListarProjectos.setText(bundle.getString("Lista_de_Projectos")); // NOI18N
        jmiListarProjectos.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiListarProjectosActionPerformed(evt);
            }
        });
        jmVer.add(jmiListarProjectos);

        jmiListarCUSemR.setText(bundle.getString("Lista_de_Casos_de_Uso_sem_Requisitos")); // NOI18N
        jmiListarCUSemR.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiListarCUSemRActionPerformed(evt);
            }
        });
        jmVer.add(jmiListarCUSemR);

        jmiListarRSemCU.setText(bundle.getString("Lista_de_Requisitos_sem_Casos_de_Uso")); // NOI18N
        jmiListarRSemCU.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiListarRSemCUActionPerformed(evt);
            }
        });
        jmVer.add(jmiListarRSemCU);
        jmVer.add(jSeparator6);

        jmiDadosUtilizadores.setText(bundle.getString("Lista_de_Utilizadores")); // NOI18N
        jmiDadosUtilizadores.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiDadosUtilizadoresActionPerformed(evt);
            }
        });
        jmVer.add(jmiDadosUtilizadores);

        jmbBarraMenu.add(jmVer);

        jmFerramentas.setText(bundle.getString("Ferramentas")); // NOI18N

        jmiGerirTiposTeste.setText(bundle.getString("Gerir_Tipos_de_Teste")); // NOI18N
        jmiGerirTiposTeste.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiGerirTiposTesteActionPerformed(evt);
            }
        });
        jmFerramentas.add(jmiGerirTiposTeste);
        jmFerramentas.add(jSeparator1);

        jmiTornarActivo.setText(bundle.getString("Marcar_como_activo")); // NOI18N
        jmiTornarActivo.setEnabled(false);
        jmiTornarActivo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiTornarActivoActionPerformed(evt);
            }
        });
        jmFerramentas.add(jmiTornarActivo);
        jmFerramentas.add(jSeparator2);

        jmiAlterarDadosPessoais.setText(bundle.getString("Alterar_dados_pessoais_...")); // NOI18N
        jmiAlterarDadosPessoais.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiAlterarDadosPessoaisActionPerformed(evt);
            }
        });
        jmFerramentas.add(jmiAlterarDadosPessoais);

        jmbBarraMenu.add(jmFerramentas);

        jmAjuda.setText(bundle.getString("Ajuda")); // NOI18N

        jmiSobre.setText(bundle.getString("Sobre")); // NOI18N
        jmiSobre.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiSobreActionPerformed(evt);
            }
        });
        jmAjuda.add(jmiSobre);

        jmbBarraMenu.add(jmAjuda);

        setJMenuBar(jmbBarraMenu);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jSplitPane, javax.swing.GroupLayout.DEFAULT_SIZE, 783, Short.MAX_VALUE)
                .addContainerGap())
            .addComponent(jtbBarraFerramentas, javax.swing.GroupLayout.DEFAULT_SIZE, 803, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jtbBarraFerramentas, javax.swing.GroupLayout.PREFERRED_SIZE, 25, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jSplitPane, javax.swing.GroupLayout.DEFAULT_SIZE, 605, Short.MAX_VALUE)
                .addContainerGap())
        );

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-811)/2, (screenSize.height-702)/2, 811, 702);
    }// </editor-fold>//GEN-END:initComponents
    private void jmiGerirTiposTesteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiGerirTiposTesteActionPerformed
        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                new JanelaGestaoTiposTeste(me, sist).setVisible(true);
            }
        });
    }//GEN-LAST:event_jmiGerirTiposTesteActionPerformed

    private void jbtnGerarRelatorioActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnGerarRelatorioActionPerformed
        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                new JanelaGerarRelatorio(me, activo).setVisible(true);
            }
        });
    }//GEN-LAST:event_jbtnGerarRelatorioActionPerformed

    private void jmiTornarActivoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiTornarActivoActionPerformed
        tornarActivo();
    }//GEN-LAST:event_jmiTornarActivoActionPerformed

    private void jbtnGravarActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnGravarActionPerformed
        gravar();
    }//GEN-LAST:event_jbtnGravarActionPerformed

    private void jmiSairActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiSairActionPerformed
        sair();
    }//GEN-LAST:event_jmiSairActionPerformed

    private void jbtnAdicionarProjectoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnAdicionarProjectoActionPerformed
        novoProjecto();
    }//GEN-LAST:event_jbtnAdicionarProjectoActionPerformed

    private void jmiSobreActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiSobreActionPerformed
        throw new UnsupportedOperationException(java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("por_implementar"));
    }//GEN-LAST:event_jmiSobreActionPerformed

    private void jmiListarProjectosActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiListarProjectosActionPerformed
        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                new JanelaListarProjectos(me, sist).setVisible(true);
            }
        });
    }//GEN-LAST:event_jmiListarProjectosActionPerformed

    private void jmiListarCUSemRActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiListarCUSemRActionPerformed
        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                new JanelaListarCUSemR(me, sist, activo).setVisible(true);
            }
        });
    }//GEN-LAST:event_jmiListarCUSemRActionPerformed

    private void jmiListarRSemCUActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiListarRSemCUActionPerformed
        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                new JanelaListarRSemCU(me, sist, activo).setVisible(true);
            }
        });
    }//GEN-LAST:event_jmiListarRSemCUActionPerformed

    private void jmiNovoProjectoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiNovoProjectoActionPerformed
        novoProjecto();
    }//GEN-LAST:event_jmiNovoProjectoActionPerformed

    private void jmiNovoRequisitoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiNovoRequisitoActionPerformed
        addNewRequisito();
    }//GEN-LAST:event_jmiNovoRequisitoActionPerformed

    private void jmiNovoCasoUsoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiNovoCasoUsoActionPerformed
        addNewCasoUso();
    }//GEN-LAST:event_jmiNovoCasoUsoActionPerformed

    private void jmiNovoTesteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiNovoTesteActionPerformed
        addNewTeste();
    }//GEN-LAST:event_jmiNovoTesteActionPerformed

    private void jmiEliminarActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiEliminarActionPerformed
        removerDados();
    }//GEN-LAST:event_jmiEliminarActionPerformed

    private void jmiGravarActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiGravarActionPerformed
        gravar();
    }//GEN-LAST:event_jmiGravarActionPerformed

    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        sair();
    }//GEN-LAST:event_formWindowClosing

    private void jmiDadosUtilizadoresActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiDadosUtilizadoresActionPerformed
        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                new JanelaDadosUtilizadores(me, sist).setVisible(true);
            }
        });
    }//GEN-LAST:event_jmiDadosUtilizadoresActionPerformed

    private void jmiAlterarDadosPessoaisActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiAlterarDadosPessoaisActionPerformed
        mostrarDadosPessoais();
    }//GEN-LAST:event_jmiAlterarDadosPessoaisActionPerformed

    private void jbtnVerDadosPessoaisActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnVerDadosPessoaisActionPerformed
        System.err.println(java.util.ResourceBundle.getBundle("requirementsmanager/resources/languages/language").getString("1:") + user.getUsername());
        mostrarDadosPessoais();
    }//GEN-LAST:event_jbtnVerDadosPessoaisActionPerformed

    private void jbtnAdicionarCasoUsoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnAdicionarCasoUsoActionPerformed
        addNewCasoUso();
    }//GEN-LAST:event_jbtnAdicionarCasoUsoActionPerformed

    private void jbtnAdicionarRequisitoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnAdicionarRequisitoActionPerformed
        addNewRequisito();
    }//GEN-LAST:event_jbtnAdicionarRequisitoActionPerformed

    private void jbtnAdicionarTesteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnAdicionarTesteActionPerformed
        addNewTeste();
    }//GEN-LAST:event_jbtnAdicionarTesteActionPerformed

    private void treeGestaoValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_treeGestaoValueChanged
        TreePath path = evt.getNewLeadSelectionPath();
        String name;
        if (path != null) {
            if (path.getPathCount() == 2) {
                jSplitPane.setRightComponent(panelGestaoProjectos);
                panelGestaoProjectos.mostarProjecto((Projecto) ((DefaultMutableTreeNode) path.getPath()[path.getPathCount() - 1]).getUserObject());
                jmiTornarActivo.setEnabled(true);
            } else if (path.getPathCount() == 4) {
                name = path.getPathComponent(path.getPathCount() - 2).toString();
                if (name.equals(TREE_CASOUSO)) {
                    jSplitPane.setRightComponent(panelGestaoCasosUso);
                    panelGestaoCasosUso.mostarCasoUso((CasoUso) ((DefaultMutableTreeNode) path.getLastPathComponent()).getUserObject());
                } else if (name.equals(TREE_REQUISITO)) {
                    jSplitPane.setRightComponent(panelGestaoRequisitos);
                    panelGestaoRequisitos.mostrarRequisito((Requisito) ((DefaultMutableTreeNode) path.getLastPathComponent()).getUserObject());
                } else if (name.equals(TREE_TESTE)) {
                    jSplitPane.setRightComponent(panelGestaoTestesAssociados);
                    panelGestaoTestesAssociados.mostarTeste((Teste) ((DefaultMutableTreeNode) path.getLastPathComponent()).getUserObject());
                }
                jmiTornarActivo.setEnabled(false);
            }
            jSplitPane.setDividerLocation(240);
        }
    }//GEN-LAST:event_treeGestaoValueChanged

    private void jbtnRemoverActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbtnRemoverActionPerformed
        removerDados();
    }//GEN-LAST:event_jbtnRemoverActionPerformed

    private void jmiLogoutActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiLogoutActionPerformed
        sist.logout();
        dispose();
        autenticacao.setVisible(true);
    }//GEN-LAST:event_jmiLogoutActionPerformed

    private void jmiMarcarActivoPopUpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiMarcarActivoPopUpActionPerformed
        tornarActivo();
    }//GEN-LAST:event_jmiMarcarActivoPopUpActionPerformed

    private void jmiEliminarPopUpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiEliminarPopUpActionPerformed
        removerDados();
    }//GEN-LAST:event_jmiEliminarPopUpActionPerformed

    private void jmiNovoTestePopUpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiNovoTestePopUpActionPerformed
        addNewTeste();
    }//GEN-LAST:event_jmiNovoTestePopUpActionPerformed

    private void jmiNovoCasoUsoPopUpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiNovoCasoUsoPopUpActionPerformed
        addNewCasoUso();
    }//GEN-LAST:event_jmiNovoCasoUsoPopUpActionPerformed

    private void jmiNovoRequisitoPopUpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiNovoRequisitoPopUpActionPerformed
        addNewRequisito();
    }//GEN-LAST:event_jmiNovoRequisitoPopUpActionPerformed

    private void jmiNovoProjectoPopUpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jmiNovoProjectoPopUpActionPerformed
        novoProjecto();
    }//GEN-LAST:event_jmiNovoProjectoPopUpActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSeparator jSeparator10;
    private javax.swing.JSeparator jSeparator11;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JToolBar.Separator jSeparator3;
    private javax.swing.JToolBar.Separator jSeparator4;
    private javax.swing.JToolBar.Separator jSeparator5;
    private javax.swing.JSeparator jSeparator6;
    private javax.swing.JSeparator jSeparator7;
    private javax.swing.JSeparator jSeparator8;
    private javax.swing.JSeparator jSeparator9;
    private javax.swing.JSplitPane jSplitPane;
    private javax.swing.JButton jbtnAdicionarCasoUso;
    private javax.swing.JButton jbtnAdicionarProjecto;
    private javax.swing.JButton jbtnAdicionarRequisito;
    private javax.swing.JButton jbtnAdicionarTeste;
    private javax.swing.JButton jbtnGerarRelatorio;
    private javax.swing.JButton jbtnGravar;
    private javax.swing.JButton jbtnRemover;
    private javax.swing.JButton jbtnVerDadosPessoais;
    private javax.swing.JMenu jmAjuda;
    private javax.swing.JMenu jmFerramentas;
    private javax.swing.JMenu jmFicheiro;
    private javax.swing.JMenu jmNovo;
    private javax.swing.JMenu jmVer;
    private javax.swing.JMenuBar jmbBarraMenu;
    private javax.swing.JMenuItem jmiAlterarDadosPessoais;
    private javax.swing.JMenuItem jmiDadosUtilizadores;
    private javax.swing.JMenuItem jmiEliminar;
    private javax.swing.JMenuItem jmiEliminarPopUp;
    private javax.swing.JMenuItem jmiGerirTiposTeste;
    private javax.swing.JMenuItem jmiGravar;
    private javax.swing.JMenuItem jmiListarCUSemR;
    private javax.swing.JMenuItem jmiListarProjectos;
    private javax.swing.JMenuItem jmiListarRSemCU;
    private javax.swing.JMenuItem jmiLogout;
    private javax.swing.JMenuItem jmiMarcarActivoPopUp;
    private javax.swing.JMenuItem jmiNovoCasoUso;
    private javax.swing.JMenuItem jmiNovoCasoUsoPopUp;
    private javax.swing.JMenuItem jmiNovoProjecto;
    private javax.swing.JMenuItem jmiNovoProjectoPopUp;
    private javax.swing.JMenuItem jmiNovoRequisito;
    private javax.swing.JMenuItem jmiNovoRequisitoPopUp;
    private javax.swing.JMenuItem jmiNovoTeste;
    private javax.swing.JMenuItem jmiNovoTestePopUp;
    private javax.swing.JMenuItem jmiSair;
    private javax.swing.JMenuItem jmiSobre;
    private javax.swing.JMenuItem jmiTornarActivo;
    private javax.swing.JPopupMenu jpumMenu;
    private javax.swing.JToolBar jtbBarraFerramentas;
    private javax.swing.JPanel panelPlacer;
    private javax.swing.JTree treeGestao;
    // End of variables declaration//GEN-END:variables
    private class PopupMouseListener extends MouseAdapter {

        @Override
        public void mousePressed(MouseEvent e) {
            tryToShowPopup(e);
        }

        @Override
        public void mouseReleased(MouseEvent e) {
            tryToShowPopup(e);
        }

        private void tryToShowPopup(MouseEvent e) {
            if (e.isPopupTrigger()) {
                jpumMenu.show(e.getComponent(), e.getX(), e.getY());
            }
        }
    }
}
