package requirementsmanager.model;

import requirementsmanager.model.listeners.GravavelListener;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;
import requirementsmanager.utils.Ligacao;

public class CasoUso implements Gravavel {

    private int id;
    private String identificador;
    private String descricao;
    private LinkedList<PontoExtensao> pontosExtensao;
    private LinkedList<Requisito> requisitos;
    private Projecto projecto;
    private boolean novo;
    private boolean removido;
    private LinkedList<GravavelListener> listeners;
    private boolean dirty;

    public CasoUso(int id, String identificador, String descricao, LinkedList<PontoExtensao> pontos, Projecto projecto, LinkedList<Requisito> requisitos, boolean novo, boolean removido) {
        this.id = id;
        this.identificador = identificador;
        this.descricao = descricao;

        this.pontosExtensao = (pontos == null) ? new LinkedList<PontoExtensao>() : pontos;
        if (projecto == null) {
            throw new IllegalArgumentException("Projecto inválido!");
        }
        this.projecto = projecto;
        this.requisitos = (requisitos == null) ? new LinkedList<Requisito>() : requisitos;

        this.novo = novo;
        this.removido = removido;
        dirty = false;
        listeners = new LinkedList<GravavelListener>();

        Logger.getLogger(CasoUso.class.getName()).log(Level.INFO, "[CASOUSO]: \"" + identificador + "\" <" + id + "> - Criar");
    }

    public CasoUso(int id, String identificador, String descricao, LinkedList<PontoExtensao> pontos, Projecto projecto, LinkedList<Requisito> requisitos) {
        this(id, identificador, descricao, pontos, projecto, requisitos, true, false);
    }

    public int getId() {
        return id;
    }

    public String getDescricao() {
        return descricao;
    }

    public void setDescricao(String descricao) {
        this.descricao = descricao;
        dirty = true;
        fireDataChanged();
    }

    public String getIdentificador() {
        return identificador;
    }

    public void setIdentificador(String identificador) {
        this.identificador = identificador;
        dirty = true;
        fireDataChanged();
    }

    public LinkedList<PontoExtensao> getPontosExtensaoValidos() {
        LinkedList<PontoExtensao> copia = new LinkedList<PontoExtensao>();
        for (PontoExtensao p : pontosExtensao) {
            if (!p.isRemoved()) {
                copia.add(p);
            }
        }

        return copia;
    }

    public LinkedList<PontoExtensao> getTodosPontosExtensao() {
        LinkedList<PontoExtensao> copia = new LinkedList<PontoExtensao>();
        for (PontoExtensao p : pontosExtensao) {
            copia.add(p);
        }

        return copia;
    }

    public boolean adicionarPontoExtensao(PontoExtensao ponto) {
        if (pontosExtensao == null) {
            pontosExtensao = new LinkedList<PontoExtensao>();
        }

        pontosExtensao.add(ponto);
        fireDataChanged();
        return true;
    }

    public boolean removerPontoExtensao(PontoExtensao ponto) {
        if (pontosExtensao != null) {
            pontosExtensao.remove(ponto);
            fireDataChanged();
            return true;
        }
        return false;
    }

    public LinkedList<Requisito> getRequisitos() {
        LinkedList<Requisito> copia = new LinkedList<Requisito>();
        for (Requisito r : requisitos) {
            copia.add(r);
        }

        return copia;
    }

    public boolean adicionarRequisito(Requisito requisito) {
        if (requisitos == null) {
            requisitos = new LinkedList<Requisito>();
        }
        requisitos.add(requisito);
        fireDataChanged();
        return true;
    }

    public boolean removerRequisito(Requisito requisito) {

        if (requisitos != null) {
            requisitos.remove(requisito);
            fireDataChanged();
            return true;
        }
        return false;
    }

    public Projecto getProjecto() {
        return projecto;
    }

    public void setProjecto(Projecto projecto) {
        if (projecto == null) {
            throw new IllegalArgumentException("Projecto inválido!");
        }
        this.projecto = projecto;
        fireDataChanged();
    }

    public boolean gravar() {
        Statement stm;
        ResultSet rs;
        boolean sucesso = false;

        if (novo && removido) {
            return true;
        }
        try {
            Logger.getLogger(CasoUso.class.getName()).log(Level.INFO, "[CASOUSO]: " + id + "[NOVO]: " + novo + " [REMOVIDO]: " + removido);
            if (novo) {
                stm = Ligacao.getConnection().createStatement();
                String insert = "INSERT INTO CasoUso(identificador, descricao, idProjecto) " +
                        "VALUES('" + identificador + "', '" + descricao + "', " + projecto.getId() + ")";

                sucesso = stm.executeUpdate(insert, Statement.RETURN_GENERATED_KEYS) == 1;

                if (sucesso) {
                    rs = stm.getGeneratedKeys();
                    if (rs.first()) {
                        id = rs.getInt(1);
                    }
                    novo = false;
                    dirty = false;
                }
            } else if (removido) {
                Logger.getLogger(CasoUso.class.getName()).log(Level.INFO, "[CASOUSO]: " + id + " - A preparar para remover");
                stm = Ligacao.getConnection().createStatement();
                String delete = "DELETE FROM CasoUso WHERE id = " + id;

                for (PontoExtensao p : pontosExtensao) {
                    p.remover();
                }
                pontosExtensao.clear();

                sucesso = stm.executeUpdate(delete) == 1;
                if (sucesso) {
                    dirty = false;
                    removido = false;
                    Logger.getLogger(CasoUso.class.getName()).log(Level.INFO, "[CASOUSO]: " + id + " - Removido com sucesso");
                }
            } else if (dirty) {

                stm = Ligacao.getConnection().createStatement();
                String update = "UPDATE CasoUso SET identificador = '" + identificador + "', descricao = '" +
                        getDescricao() + "', idProjecto = " + projecto.getId() + " WHERE id = " + id;

                sucesso = stm.executeUpdate(update) == 1;

                if (sucesso) {
                    dirty = false;
                }
            }
        } catch (SQLException ex) {
            Logger.getLogger(CasoUso.class.getName()).log(Level.SEVERE, null, ex);
        }
        return sucesso;
    }

    @Override
    public String toString() {
        return identificador;
    }

    public boolean isNew() {
        return novo;
    }

    public boolean isRemoved() {
        return removido;
    }

    public void fireDataChanged() {
        for (GravavelListener gl : listeners) {
            gl.dataChanged(this);
        }
    }

    public void addListener(GravavelListener g) {
        if (listeners == null) {
            listeners = new LinkedList<GravavelListener>();
        }

        listeners.add(g);
    }

    public void removeListener(GravavelListener g) {
        if (listeners != null) {
            listeners.remove(g);
        }
    }

    public void marcarRemocao() {
        removido = true;
        fireDataChanged();
    }
}

