package requirementsmanager.model;

import java.sql.SQLException;
import java.sql.Statement;
import java.util.GregorianCalendar;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;
import requirementsmanager.utils.Ligacao;

public class Gestor extends Utilizador {

    private LinkedList<Projecto> projectos;

    public Gestor(int id, String nome, String endereco, String codigoExtra, CodigoPostal codigo, String email, String msn, String icq, String sexo, GregorianCalendar dataNascimento, String telefone, String telemovel, String username, String password, LinkedList<Projecto> projectos, boolean novo, boolean removido) {
        super(id, nome, endereco, codigoExtra, codigo, email, msn, icq, sexo, dataNascimento, telefone, telemovel, username, password, Utilizador.GESTOR, novo, removido);
        this.projectos = (projectos == null) ? new LinkedList<Projecto>() : projectos;
    }

    public Gestor(int id, String nome, String endereco, String codigoExtra, CodigoPostal codigo, String email, String msn, String icq, String sexo, GregorianCalendar dataNascimento, String telefone, String telemovel, String username, String password, LinkedList<Projecto> projectos) {
        this(id, nome, endereco, codigoExtra, codigo, email, msn, icq, sexo, dataNascimento, telefone, telemovel, username, password, projectos, false, false);
    }

    public LinkedList<Projecto> getProjectosValidos() {
        LinkedList<Projecto> copia = new LinkedList<Projecto>();
        for (Projecto p : projectos) {
            if (!p.isRemoved()) {
                copia.add(p);
            }
        }

        return copia;
    }
    
    public LinkedList<Projecto> getTodosProjectos() {
        LinkedList<Projecto> copia = new LinkedList<Projecto>();
        for (Projecto p : projectos) {
                copia.add(p);
        }

        return copia;
    }
    
    public Projecto getProjecto(int id) {
        for(Projecto p : projectos) {
            if(p.getId() == id) {
                return p;
            }
        }
        
        return null;
    }

    public boolean adicionarProjecto(Projecto projecto) {
        if (projectos == null) {
            projectos = new LinkedList<Projecto>();
        }
        
        boolean resultado = projectos.add(projecto);
        if (resultado && projecto.isNew()) {
            fireDataChanged();
            setDirty(true);
        }
        return resultado;
    }

    public boolean removerProjecto(Projecto projecto) {
        boolean resultado = false;

        if (projectos != null) {
            resultado = projectos.remove(projecto);
            Logger.getLogger(Gestor.class.getName()).log(Level.INFO, "[GESTOR]: " + getId() + " - Remover [PROJECTO]: " + projecto.getId());
            if (resultado) {
                fireDataChanged();
                setDirty(true);
            }
        }
        return resultado;
    }

    @Override
    public boolean gravar() {
        if(isNew() && isRemoved()) {
            return true;
        }
        if (isRemoved()) {
            try {
                Statement stm = Ligacao.getConnection().createStatement();
                stm.executeUpdate("DELETE FROM Gerir WHERE idGestor = " + getId());
            } catch (SQLException ex) {
                Logger.getLogger(Gestor.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        return super.gravar();
    }
}

