package requirementsmanager.model;

import requirementsmanager.model.listeners.GravavelListener;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;
import requirementsmanager.utils.Ligacao;

public class PontoExtensao implements Gravavel {

    private int id;
    private CasoUso casoUso;
    private String nome;
    private boolean novo;
    private boolean removido;
    private boolean dirty;
    private LinkedList<GravavelListener> listeners;

    public PontoExtensao(int id, String nome, CasoUso casoUso, boolean novo, boolean removido) {
        this.id = id;
        this.nome = nome;
        if (casoUso == null) {
            throw new IllegalArgumentException("Caso de uso inválido");
        }
        this.casoUso = casoUso;

        this.novo = novo;
        this.removido = removido;
        dirty = false;

        listeners = new LinkedList<GravavelListener>();
        Logger.getLogger(PontoExtensao.class.getName()).log(Level.INFO, "[PONTOEXTENSAO]: \"" + nome + "\" <" + id + "> - Criar");
    }

    public PontoExtensao(int id, String nome, CasoUso casoUso) {
        this(id, nome, casoUso, true, false);
    }

    public int getId() {
        return id;
    }

    public CasoUso getCasoUso() {
        return casoUso;
    }

    public void setCasoUso(CasoUso caso) {
        if (caso == null) {
            throw new IllegalArgumentException("Caso de uso inválido");
        }
        casoUso = caso;
        dirty = true;
        fireDataChanged();
    }

    public String getNome() {
        return nome;
    }

    public void setNome(String nome) {
        this.nome = nome;
        dirty = true;
        fireDataChanged();
    }

    public boolean gravar() {
        Statement stm;
        ResultSet rs;
        boolean sucesso = false;
        if(novo && removido) {
            return true;
        }
        try {
            if (novo) {
                stm = Ligacao.getConnection().createStatement();
                String insert = "INSERT INTO PontoExtensao(idCasoUso, nome) VALUES (" +
                        casoUso.getId() + ", '" + nome + "')";

                sucesso = stm.executeUpdate(insert, Statement.RETURN_GENERATED_KEYS) == 1;

                if (sucesso) {
                    rs = stm.getGeneratedKeys();
                    if (rs.first()) {
                        id = rs.getInt(1);
                    }
                    novo = false;
                    dirty = false;
                }

            }else if (removido) {
                sucesso = remover();
            } else if (dirty) {
                stm = Ligacao.getConnection().createStatement();
                String update = "UPDATE PontoExtensao SET idCasoUso = " + casoUso.getId() +
                        ", nome = '" + nome + "' WHERE id = " + id;

                sucesso = stm.executeUpdate(update) == 1;

                if (sucesso) {
                    dirty = false;
                }
            }
        } catch (SQLException ex) {
            Logger.getLogger(Utilizador.class.getName()).log(Level.SEVERE, null, ex);
        }
        return sucesso;
    }

    public boolean remover() {
        Statement stm;
        boolean sucesso = false;
        try {
            stm = Ligacao.getConnection().createStatement();

            String delete = "DELETE FROM PontoExtensao WHERE id = " + id;

            sucesso = stm.executeUpdate(delete) == 1;

            if (sucesso) {
                dirty = false;
                removido = false;
            }
        } catch (SQLException ex) {
            Logger.getLogger(Utilizador.class.getName()).log(Level.SEVERE, null, ex);
        }

        return sucesso;
    }

    public boolean isNew() {
        return novo;
    }

    public boolean isRemoved() {
        return removido;
    }

    public void fireDataChanged() {
        for (GravavelListener gl : listeners) {
            gl.dataChanged(this);
        }
    }

    public void addListener(GravavelListener g) {
        if (listeners == null) {
            listeners = new LinkedList<GravavelListener>();
        }

        listeners.add(g);
    }

    public void removeListener(GravavelListener g) {
        if (listeners != null) {
            listeners.remove(g);
        }
    }

    public void marcarRemocao() {
        removido = true;
    }

    @Override
    public String toString() {
        return nome;
    }
}

