package requirementsmanager.model;

import requirementsmanager.model.listeners.GravavelListener;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;
import requirementsmanager.utils.Ligacao;

public class Projecto implements Gravavel {

    private int id;
    private String nome;
    private String descricao;
    private LinkedList<Requisito> requisitos;
    private LinkedList<CasoUso> casosUso;
    private LinkedList<Teste> testes;
    private LinkedList<Gestor> gestores;
    private boolean novo;
    private boolean removido;
    private boolean dirty;
    private LinkedList<GravavelListener> listeners;

    public Projecto(int id, String nome, String descricao, LinkedList<Requisito> requisitos, LinkedList<CasoUso> casosUso, LinkedList<Teste> testes, LinkedList<Gestor> gestores, boolean novo, boolean removido) {
        this.id = id;
        this.nome = nome;
        this.descricao = descricao;
        this.requisitos = (requisitos == null ? new LinkedList<Requisito>() : requisitos);
        this.gestores = (gestores == null ? new LinkedList<Gestor>() : gestores);
        this.casosUso = (casosUso == null ? new LinkedList<CasoUso>() : casosUso);
        this.testes = (testes == null ? new LinkedList<Teste>() : testes);

        this.novo = novo;
        this.removido = removido;
        dirty = false;
        listeners = new LinkedList<GravavelListener>();
        Logger.getLogger(Projecto.class.getName()).log(Level.INFO, "[PROJECTO]: \"" + nome + "\" <" + id + "> - Criar");
    }

    public Projecto(int id, String nome, String descricao, LinkedList<Requisito> requisitos, LinkedList<CasoUso> casosUso, LinkedList<Teste> testes, LinkedList<Gestor> gestores) {
        this(id, nome, descricao, requisitos, casosUso, testes, gestores, true, false);
    }

    public void setOrfao() {
        removerTodosGestores();
    }

    public String getDescricao() {
        return descricao;
    }

    public void setDescricao(String val) {
        this.descricao = val;
        fireDataChanged();
    }

    public String getNome() {
        return nome;
    }

    public void setNome(String val) {
        this.nome = val;
        fireDataChanged();
    }

    public int getId() {
        return id;
    }

    public boolean adicionarTeste(Teste teste) {
        if (testes == null) {
            testes = new LinkedList<Teste>();
        }
        testes.add(teste);
        fireDataChanged();
        return true;
    }

    public boolean removerTeste(Teste teste) {
        if (testes != null) {
            testes.remove(teste);
            fireDataChanged();
            return true;
        }
        return false;
    }

    public void removerTodosTestes() {
        if (testes != null) {
            for (Teste t : testes) {
                t.setProjecto(null);
            }
            testes.clear();
            fireDataChanged();
        }
    }

    public boolean adicionarCasoUso(CasoUso caso) {
        if (casosUso == null) {
            casosUso = new LinkedList<CasoUso>();
        }
        casosUso.add(caso);
        fireDataChanged();
        return true;
    }

    public boolean removerCasoUso(CasoUso casoUso) {
        if (casosUso != null) {
            casosUso.remove(casoUso);
            fireDataChanged();
            return true;
        }
        return false;
    }

    public void removerTodosCasosUso() {
        if (casosUso != null) {
            for (CasoUso c : casosUso) {
                c.setProjecto(null);
            }
            casosUso.clear();
            fireDataChanged();
        }
    }

    public boolean adicionarRequisito(Requisito requisito) {
        if (requisitos == null) {
            requisitos = new LinkedList<Requisito>();
        }
        requisitos.add(requisito);
        fireDataChanged();
        return true;
    }

    boolean removerRequisito(Requisito requisito) {
        if (requisitos != null) {
            requisitos.remove(requisito);
            fireDataChanged();
            return true;
        }
        return false;
    }

    public void removerTodosRequisitos() {
        if (requisitos != null) {
            for (Requisito r : requisitos) {
                r.setProjecto(null);
            }
            requisitos.clear();
            fireDataChanged();
        }
    }

    public boolean adicionarGestor(Gestor gestor) {
        if (gestores == null) {
            gestores = new LinkedList<Gestor>();
        }
        gestores.add(gestor);
        fireDataChanged();
        return true;
    }

    public boolean removerGestor(Gestor gestor) {
        if (gestores != null) {
            gestores.remove(gestor);
            fireDataChanged();
            return true;
        }
        return false;
    }

    public void removerTodosGestores() {
        if (gestores != null) {
            for (Gestor g : gestores) {
                g.removerProjecto(this);
            }
            gestores.clear();
            fireDataChanged();
        }
    }

    public LinkedList<CasoUso> getListaCasosUsoValidos() {
        LinkedList<CasoUso> copia = new LinkedList<CasoUso>();
        for (CasoUso c : casosUso) {
            if (!c.isRemoved()) {
                copia.add(c);
            }
        }

        return copia;
    }

    public LinkedList<Requisito> getListaRequisitosValidos() {
        LinkedList<Requisito> copia = new LinkedList<Requisito>();
        for (Requisito r : requisitos) {
            if (!r.isRemoved()) {
                copia.add(r);
            }
        }

        return copia;
    }

    public LinkedList<Teste> getListaTestesValidos() {
        LinkedList<Teste> copia = new LinkedList<Teste>();
        for (Teste t : testes) {
            if (!t.isRemoved()) {
                copia.add(t);
            }
        }

        return copia;
    }

    public LinkedList<Gestor> getListaGestores() {
        LinkedList<Gestor> copia = new LinkedList<Gestor>();
        for (Gestor g : gestores) {
            if (!g.isRemoved()) {
                copia.add(g);
            }
        }

        return copia;
    }

    public CasoUso getCasoUso(int idCasoUso) {
        for (CasoUso c : casosUso) {
            if (c.getId() == idCasoUso) {
                return c;
            }
        }

        return null;
    }

    public Requisito getRequisito(int idRequisito) {
        for (Requisito r : requisitos) {
            if (r.getId() == idRequisito) {
                return r;
            }
        }

        return null;
    }

    public Teste getTeste(int idTeste) {
        for (Teste t : testes) {
            if (t.getId() == idTeste) {
                return t;
            }
        }

        return null;
    }

    public Gestor getGestor(int idGestor) {
        for (Gestor g : gestores) {
            if (g.getId() == idGestor) {
                return g;
            }
        }

        return null;
    }

    public LinkedList<CasoUso> getListaTodosCasosUso() {
        LinkedList<CasoUso> copia = new LinkedList<CasoUso>();
        for (CasoUso c : casosUso) {
            copia.add(c);
        }

        return copia;
    }

    public LinkedList<Requisito> getListaTodosRequisitos() {
        LinkedList<Requisito> copia = new LinkedList<Requisito>();
        for (Requisito r : requisitos) {
            copia.add(r);
        }

        return copia;
    }

    public LinkedList<Teste> getListaTodosTestes() {
        LinkedList<Teste> copia = new LinkedList<Teste>();
        for (Teste t : testes) {
            copia.add(t);
        }

        return copia;
    }

    public boolean gravar() {
        Statement stm;
        ResultSet rs;
        boolean sucesso = false;

        if(novo && removido) {
            return true;
        }
       try {
            if (novo) {
                stm = Ligacao.getConnection().createStatement();
                String insert = "INSERT INTO Projecto(nome, descricao) VALUES('" + nome + "', '" + descricao + "')";

                sucesso = stm.executeUpdate(insert, Statement.RETURN_GENERATED_KEYS) == 1;

                if (sucesso) {
                    rs = stm.getGeneratedKeys();
                    if (rs.first()) {
                        id = rs.getInt(1);
                    }
                    novo = false;
                    dirty = false;
                    Logger.getLogger(Projecto.class.getName()).log(Level.INFO, "[PROJECTO]: " + id + " - Inserido");
                }
            } else if (removido) {
                stm = Ligacao.getConnection().createStatement();
                String delete = "DELETE FROM Projecto WHERE id = " + id;

                sucesso = stm.executeUpdate(delete) == 1;

                if (sucesso) {
                    removido = false;
                    Logger.getLogger(Projecto.class.getName()).log(Level.INFO, "[PROJECTO]: " + id + " - Removido");
                }
            } else if (dirty) {
                stm = Ligacao.getConnection().createStatement();
                String update = "UPDATE Projecto SET nome = '" + nome + "', descricao = '" + descricao + "' WHERE id = " + id;

                sucesso = stm.executeUpdate(update) == 1;

                if (sucesso) {
                    dirty = false;
                    Logger.getLogger(Projecto.class.getName()).log(Level.INFO, "[PROJECTO]: " + id + " - Actualizado");
                }
            }
        } catch (SQLException ex) {
            Logger.getLogger(Utilizador.class.getName()).log(Level.SEVERE, null, ex);
        }
        return sucesso;
    }

    public boolean isNew() {
        return novo;
    }

    public boolean isRemoved() {
        return removido;
    }

    public void fireDataChanged() {
        for (GravavelListener gl : listeners) {
            gl.dataChanged(this);
        }
    }

    public void addListener(GravavelListener g) {
        if (listeners == null) {
            listeners = new LinkedList<GravavelListener>();
        }

        listeners.add(g);
    }

    public void removeListener(GravavelListener g) {
        if (listeners != null) {
            listeners.remove(g);
        }
    }

    public void marcarRemocao() {
        removido = true;

        if (casosUso != null) {
            for (CasoUso c : casosUso) {
                c.marcarRemocao();
            }
        }

        if (requisitos != null) {
            for (Requisito r : requisitos) {
                r.marcarRemocao();
            }
        }

        if (testes != null) {
            for (Teste t : testes) {
                t.marcarRemocao();
            }
        }

        fireDataChanged();
    }

    @Override
    public String toString() {
        return nome;
    }
}
