package requirementsmanager.model;

import requirementsmanager.model.listeners.GravavelListener;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;
import requirementsmanager.utils.Ligacao;

public class Requisito implements Gravavel {

    public static final int ORIGEM_EQUIPA = 1;
    public static final int ORIGEM_CLIENTE = 2;
    public static final int RISCO_ALTO = 1;
    public static final int RISCO_BAIXO = 2;
    public static final int PRIORIDADE_ALTA = 1;
    public static final int PRIORIDADE_MEDIA = 2;
    public static final int PRIORIDADE_BAIXA = 3;
    private int id;
    private String identificador;
    private String descricao;
    private int origem;
    private int risco;
    private int prioridade;
    private LinkedList<CasoUso> casosUso;
    private LinkedList<Teste> testes;
    private Projecto projecto;
    private boolean novo;
    private boolean removido;
    private LinkedList<GravavelListener> listeners;
    private boolean dirty;

    public Requisito(int id, String identificador, int origem, int risco, int prioridade, String descricao, LinkedList<CasoUso> casosUso, LinkedList<Teste> testes, Projecto projecto, boolean novo, boolean removido) {
        this.id = id;
        this.identificador = identificador;
        this.descricao = descricao;
        this.casosUso = (casosUso == null ? new LinkedList<CasoUso>() : casosUso);
        this.testes = (testes == null ? new LinkedList<Teste>() : testes);

        if (projecto == null) {
            throw new IllegalArgumentException("Projecto inválido");
        }
        this.projecto = projecto;

        this.novo = novo;
        this.removido = removido;
        listeners = new LinkedList<GravavelListener>();

        setOrigem(origem);
        setRisco(risco);
        setPrioridade(prioridade);

        dirty = false;
    }

    public Requisito(int id, String identificador, int origem, int risco, int prioridade, String descricao, LinkedList<CasoUso> casosUso, LinkedList<Teste> testes, Projecto projecto) {
        this(id, identificador, origem, risco, prioridade, descricao, casosUso, testes, projecto, true, false);
    }

    public Requisito(int id, String identificador, int origem, int risco, int prioridade, String descricao, Projecto projecto) {
        this(id, identificador, origem, risco, prioridade, descricao, null, null, projecto, true, false);
    }

    public int getId() {
        return id;
    }

    public String getDescricao() {
        return descricao;
    }

    public void setDescricao(String val) {
        this.descricao = val;
        dirty = true;
        fireDataChanged();
    }

    public String getIdentificador() {
        return identificador;
    }

    public void setIdentificador(String val) {
        this.identificador = val;
        dirty = true;
        fireDataChanged();
    }

    public boolean adicionarCasoUso(CasoUso caso) {
        if (casosUso == null) {
            casosUso = new LinkedList<CasoUso>();
        }
        casosUso.add(caso);
        fireDataChanged();
        return true;
    }

    public boolean removerCasoUso(CasoUso caso) {
        if (casosUso != null) {
            casosUso.remove(caso);
                fireDataChanged();
             return true;   
            }
        return false;
    }

    public LinkedList<CasoUso> getCasosUso() {
        LinkedList<CasoUso> copia = new LinkedList<CasoUso>();
        for (CasoUso c : casosUso) {
            copia.add(c);
        }

        return copia;
    }

    public LinkedList<Teste> getTestes() {
        LinkedList<Teste> copia = new LinkedList<Teste>();
        for (Teste t : testes) {
            copia.add(t);
        }

        return copia;
    }

    public boolean adicionarTeste(Teste teste) {
        if (testes == null) {
            testes = new LinkedList<Teste>();
        }
        testes.add(teste);
        fireDataChanged();
        return true;
    }

    public boolean removerTeste(Teste teste) {
        if (testes != null) {
testes.remove(teste);
                fireDataChanged();
                return true;
        }
        return false;
    }

    public int getOrigem() {
        return origem;
    }

    public void setOrigem(int origem) {
        if (origem != ORIGEM_CLIENTE && origem != ORIGEM_EQUIPA) {
            throw new IllegalArgumentException("Origem inválida");
        }
        this.origem = origem;
        dirty = true;
        fireDataChanged();
    }

    public int getPrioridade() {
        return prioridade;
    }

    public void setPrioridade(int prioridade) {
        if (prioridade != PRIORIDADE_ALTA && prioridade != PRIORIDADE_BAIXA && prioridade != PRIORIDADE_MEDIA) {
            throw new IllegalArgumentException("Prioridade inválida");
        }
        this.prioridade = prioridade;
        dirty = true;
        fireDataChanged();
    }

    public int getRisco() {
        return risco;
    }

    public void setRisco(int risco) {
        if (risco != RISCO_ALTO && risco != RISCO_BAIXO) {
            throw new IllegalArgumentException("Risco inválido");
        }
        this.risco = risco;
        dirty = true;
        fireDataChanged();
    }

    public Projecto getProjecto() {
        return projecto;
    }

    public void setProjecto(Projecto projecto) {
        if (projecto == null) {
            throw new IllegalArgumentException("Projecto inválido");
        }
        this.projecto = projecto;
        dirty = true;
        fireDataChanged();
    }

    public boolean gravar() {
        Statement stm;
        ResultSet rs;
        boolean sucesso = false;

        if(novo && removido) {
            return true;
        }
        try {
            if (novo) {
                stm = Ligacao.getConnection().createStatement();
                String insert = "INSERT INTO Requisito(identificador, origem, risco, prioridade, " +
                        "descricao, idProjecto) VALUES('" + identificador + "', " + origem + ", " +
                        risco + ", " + prioridade + ", '" + descricao + "', " + projecto.getId() + ")";

                sucesso = stm.executeUpdate(insert, Statement.RETURN_GENERATED_KEYS) == 1;

                if (sucesso) {
                    rs = stm.getGeneratedKeys();
                    if (rs.first()) {
                        id = rs.getInt(1);
                    }
                    novo = false;
                    dirty = false;
                }
            } else if (removido) {
                String delete = "DELETE FROM Requisito WHERE id = " + id;
                stm = Ligacao.getConnection().createStatement();

                sucesso = stm.executeUpdate(delete) == 1;

                if (sucesso) {
                    removido = false;
                    dirty = false;
                }

            } else if (dirty) {
                stm = Ligacao.getConnection().createStatement();
                String update = "UPDATE Requisito SET identificador = '" + identificador + "', origem = " +
                        origem + ", risco = " + risco + ", prioridade = " + prioridade + ", descricao = '" +
                        descricao + "', idProjecto = " + projecto.getId() + " WHERE id = " + id;

                sucesso = stm.executeUpdate(update) == 1;

                if (sucesso) {
                    dirty = false;
                }
            }
        } catch (SQLException ex) {
            Logger.getLogger(Utilizador.class.getName()).log(Level.SEVERE, null, ex);
        }
        return sucesso;
    }

    @Override
    public String toString() {
        return identificador;
    }

    public boolean isNew() {
        return novo;
    }

    public boolean isRemoved() {
        return removido;
    }

    public void fireDataChanged() {
        for (GravavelListener gl : listeners) {
            gl.dataChanged(this);
        }
    }

    public void addListener(GravavelListener g) {
        if (listeners == null) {
            listeners = new LinkedList<GravavelListener>();
        }

        listeners.add(g);
    }

    public void removeListener(GravavelListener g) {
        if (listeners != null) {
            listeners.remove(g);
        }
    }

    public void marcarRemocao() {
        removido = true;
        fireDataChanged();
    }
}

