package requirementsmanager.model;

import requirementsmanager.model.listeners.GravavelListener;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.Collection;
import java.util.GregorianCalendar;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.LinkedList;
import requirementsmanager.utils.Ligacao;

public class Sistema implements GravavelListener {

    /**/
    private Utilizador autenticado;
    private Hashtable<Integer, CodigoPostal> codigosPostais;
    private Hashtable<Integer, TipoTeste> tiposTeste;
    /*Estruturas de Dados*/
    private Gestor gestor;
    private LinkedList<Utilizador> listaUtilizadores;
    private LinkedList<Satisfazer> satisfacoes;
    private LinkedList<Gerir> gestores;
    private LinkedList<Associado> associados;
    private LinkedList<Projecto> listaProjectos;
    /*Sistema de persistencia*/
    private LinkedList<Gravavel> novasEntidadesAssociativas;
    private LinkedList<Gravavel> entidadesAssociativasARemover;
    private LinkedList<Gravavel> utilizadoresAGravar;
    private LinkedList<Gravavel> tiposTesteAGravar;

    public Sistema() {
        codigosPostais = new Hashtable<Integer, CodigoPostal>();
        tiposTeste = new Hashtable<Integer, TipoTeste>();
        listaUtilizadores = new LinkedList<Utilizador>();
        satisfacoes = new LinkedList<Satisfazer>();
        gestores = new LinkedList<Gerir>();
        associados = new LinkedList<Associado>();
        listaProjectos = new LinkedList<Projecto>();

        novasEntidadesAssociativas = new LinkedList<Gravavel>();
        entidadesAssociativasARemover = new LinkedList<Gravavel>();
        utilizadoresAGravar = new LinkedList<Gravavel>();
        tiposTesteAGravar = new LinkedList<Gravavel>();

        try {
            Statement stm = Ligacao.getConnection().createStatement();
            ResultSet rs = stm.executeQuery("SELECT * FROM CodigoPostal");
            int id;
            while (rs.next()) {
                id = rs.getInt("id");
                codigosPostais.put(id, new CodigoPostal(id, rs.getString("codigo"), rs.getString("localidade")));
            }

            stm = Ligacao.getConnection().createStatement();
            rs = stm.executeQuery("SELECT * FROM TipoTeste");
            while (rs.next()) {
                id = rs.getInt("id");
                tiposTeste.put(id, new TipoTeste(id, rs.getString("nome"), rs.getString("descricao"), false, false));
            }
        } catch (SQLException ex) {
            Logger.getLogger(Sistema.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /**
     * 
     * @param username nome de utilizador a pesquisar.
     * @param password password do utilizador sem codificacao.
     * @return Utilizador autenticado ou <em>null</em> caso as credenciais sejam
     * invalidas
     */
    public Utilizador autenticar(String username, String password) {
        try {
            Statement stmt = Ligacao.getConnection().createStatement();
            ResultSet rs = stmt.executeQuery("SELECT * FROM utilizador WHERE u_username LIKE '" + username + "' AND u_password LIKE '" + Utilizador.md5EncodeString(password) + "'");
            if (rs.first()) {
                GregorianCalendar nascimento = null;
                Date tempDate;
                if ((tempDate = rs.getDate("dataNascimento")) != null) {
                    nascimento = new GregorianCalendar();
                    nascimento.setTime(tempDate);
                }

                if (rs.getString("tipo").equals("G")) {
                    autenticado = gestor = new Gestor(rs.getInt("id"), rs.getString("nome"), rs.getString("endereco"), rs.getString("codExtra"), codigosPostais.get(new Integer(rs.getInt("codPostal"))), rs.getString("email"), rs.getString("msn"), rs.getString("icq"), rs.getString("sexo"), nascimento, rs.getString("telefone"), rs.getString("telemovel"), rs.getString("u_username"), rs.getString("u_password"), null);
                    preencherModelo();
                } else {
                    autenticado = new Administrador(rs.getInt("id"), rs.getString("nome"), rs.getString("endereco"), rs.getString("codExtra"), codigosPostais.get(new Integer(rs.getInt("codPostal"))), rs.getString("email"), rs.getString("msn"), rs.getString("icq"), rs.getString("sexo"), nascimento, rs.getString("telefone"), rs.getString("telemovel"), rs.getString("u_username"), rs.getString("u_password"));
                }
                autenticado.addListener(this);
                listaUtilizadores.add(autenticado);
                preencherListaUtilizadores();
                return autenticado;
            }
        } catch (SQLException ex) {
            Logger.getLogger(Sistema.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    /**
     * 
     * @return
     */
    public LinkedList<Projecto> getProjectos(boolean reiniciar) {
        LinkedList<Projecto> copia = new LinkedList<Projecto>();

        if (listaProjectos.isEmpty() || reiniciar) {
            actualizarListaProjectos();
        }

        for (Projecto p : listaProjectos) {
            if (!p.isRemoved()) {
                copia.add(p);
            }
        }
        return copia;
    }

    /**
     * 
     */
    private void actualizarListaProjectos() {
        Projecto projectoAuxiliar;
        try {
            
            PreparedStatement psGerir = Ligacao.getConnection().prepareStatement("SELECT idGestor FROM Gerir WHERE idProjecto = ?");
            ResultSet rsGerir;
            
            Statement stmt = Ligacao.getConnection().createStatement();
            ResultSet rs = stmt.executeQuery("SELECT * FROM Projecto");
            while (rs.next()) {
                projectoAuxiliar = new Projecto(rs.getInt("id"), rs.getString("nome"), rs.getString("descricao"), null, null, null, null, false, false);
                if (gestor != null && gestor.getProjecto(projectoAuxiliar.getId()) == null && !projectoExisteNaLista(projectoAuxiliar.getId())) {
                    listaProjectos.add(projectoAuxiliar);
                }
                
                psGerir.setInt(1, projectoAuxiliar.getId());
                rsGerir = psGerir.executeQuery();
                while(rsGerir.next()) {
                    int id = rsGerir.getInt("idGestor");
                    for(Utilizador u : listaUtilizadores) {
                        if(u.getId() == id) {
                            projectoAuxiliar.adicionarGestor((Gestor)u);
                            ((Gestor)u).adicionarProjecto(projectoAuxiliar);
                             gestores.add(new Gerir(projectoAuxiliar, (Gestor)u, false));
                        }
                    }
                }
                
            }
        } catch (SQLException ex) {
            Logger.getLogger(Sistema.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    
    private boolean projectoExisteNaLista(int id) {
        for(Projecto p : listaProjectos) {
            if(p.getId() == id) {
                return true;
            }
        }
        return false;
    }

    public LinkedList<Projecto> getProjectosOrfaos() {
        LinkedList<Projecto> copia = new LinkedList<Projecto>();

        if (listaProjectos.isEmpty()) {
            actualizarListaProjectos();
        }

        for (Projecto p : listaProjectos) {
            if (p.getListaGestores().isEmpty() && !p.isRemoved()) {
                copia.add(p);
            }
        }
        return copia;
    }

    /**
     * 
     * @param gestor
     * @param projecto
     * @return
     */
    public boolean associar(Gestor gestor, Projecto projecto) {
        gestor.adicionarProjecto(projecto);
        projecto.adicionarGestor(gestor);

        Gerir g = new Gerir(projecto, gestor);
        novasEntidadesAssociativas.add(g);
        return gestores.add(g);
    }

    /**
     * 
     * @param casoUso
     * @param requisito
     * @return
     */
    public boolean associar(CasoUso casoUso, Requisito requisito) {
        casoUso.adicionarRequisito(requisito);
        requisito.adicionarCasoUso(casoUso);

        Associado a = new Associado(casoUso, requisito);
        novasEntidadesAssociativas.add(a);
        return associados.add(a);
    }

    /**
     * 
     * @param teste
     * @param requisito
     * @return
     */
    public boolean associar(Teste teste, Requisito requisito) {
        requisito.adicionarTeste(teste);
        teste.adicionarRequisito(requisito);

        Satisfazer s = new Satisfazer(teste, requisito);
        novasEntidadesAssociativas.add(s);
        return satisfacoes.add(s);
    }

    /**
     * 
     * @param casoUso
     * @return
     */
    public boolean associarTodos(CasoUso casoUso) {
        boolean success = true;

        LinkedList<Requisito> l = casoUso.getRequisitos();
        for (Requisito r : casoUso.getProjecto().getListaRequisitosValidos()) {
            if (!l.contains(r)) {
                r.adicionarCasoUso(casoUso);
                casoUso.adicionarRequisito(r);
                Associado a = new Associado(casoUso, r);
                novasEntidadesAssociativas.add(a);
                if (associados.add(a) == false) {
                    success = false;
                }
            }
        }

        return success;
    }

    /**
     * 
     * @param projecto
     * @return
     */
    public boolean associarTodos(Projecto projecto) {
        boolean success = true;

        LinkedList<Gestor> l = projecto.getListaGestores();
        for (Utilizador u : getUtilizadores()) {
            if (!l.contains(u)) {
                if (u.getTipo() == Utilizador.GESTOR) {
                    Gestor gest = (Gestor) u;
                    gest.adicionarProjecto(projecto);
                    projecto.adicionarGestor(gest);

                    Gerir g = new Gerir(projecto, gest);
                    novasEntidadesAssociativas.add(g);
                    if (gestores.add(g) == false) {
                        success = false;
                    }
                }
            }
        }

        return success;
    }

    /**
     * 
     * @param requisito
     * @return
     */
    public boolean associarTodosCasosUso(Requisito requisito) {
        boolean success = true;

        LinkedList<CasoUso> l = requisito.getCasosUso();
        for (CasoUso c : requisito.getProjecto().getListaCasosUsoValidos()) {
            if (!l.contains(c)) {
                c.adicionarRequisito(requisito);
                requisito.adicionarCasoUso(c);
                Associado a = new Associado(c, requisito);
                novasEntidadesAssociativas.add(a);
                if (associados.add(a) == false) {
                    success = false;
                }
            }
        }
        return success;
    }

    /**
     * 
     * @param requisito
     * @return
     */
    public boolean associarTodosTestes(Requisito requisito) {
        boolean success = true;

        LinkedList<Teste> l = requisito.getTestes();
        for (Teste t : requisito.getProjecto().getListaTestesValidos()) {
            if (!l.contains(t)) {
                t.adicionarRequisito(requisito);
                requisito.adicionarTeste(t);
                Satisfazer s = new Satisfazer(t, requisito);
                novasEntidadesAssociativas.add(s);
                if (satisfacoes.add(s) == false) {
                    success = false;
                }
            }
        }
        return success;
    }

    /**
     * 
     * @param gestor
     * @param projecto
     * @return
     */
    public boolean desassociar(Gestor gestor, Projecto projecto) {
        gestor.removerProjecto(projecto);
        projecto.removerGestor(gestor);
        for (Gerir g : gestores) {
            if (g.getGestor().equals(gestor) && g.getProjecto().equals(projecto)) {
                g.marcarRemocao();
                entidadesAssociativasARemover.add(g);
                return gestores.remove(g);
            }
        }
        return false;
    }

    /**
     * 
     * @param casoUso
     * @param requisito
     * @return
     */
    public boolean desassociar(CasoUso casoUso, Requisito requisito) {
        casoUso.removerRequisito(requisito);
        requisito.removerCasoUso(casoUso);
        for (Associado a : associados) {
            if (a.getCasoUso().equals(casoUso) && a.getRequisito().equals(requisito)) {
                a.marcarRemocao();
                entidadesAssociativasARemover.add(a);
                return associados.remove(a);
            }
        }
        return false;
    }

    /**
     * 
     * @param teste
     * @param requisito
     * @return
     */
    public boolean desassociar(Teste teste, Requisito requisito) {
        teste.removerRequisito(requisito);
        requisito.removerTeste(teste);
        for (Satisfazer s : satisfacoes) {
            if (s.getTeste().equals(teste) && s.getRequisito().equals(requisito)) {
                s.marcarRemocao();
                entidadesAssociativasARemover.add(s);
                return satisfacoes.remove(s);
            }
        }
        return false;
    }

    /**
     * 
     * @param projecto
     * @return
     */
    public boolean desassociarTodos(Projecto projecto) {
        boolean success = true;
        LinkedList<Gerir> remover = new LinkedList<Gerir>();

        for (Gerir g : gestores) {
            if (g.getProjecto().equals(projecto)) {
                g.getGestor().removerProjecto(projecto);
                projecto.removerGestor(g.getGestor());
                g.marcarRemocao();
                entidadesAssociativasARemover.add(g);
                remover.add(g);
            }
        }

        for (Gerir g : remover) {
            if (gestores.remove(g) == false) {
                success = false;
            }
        }

        return success;
    }

    /**
     * 
     * @param casoUso
     * @return
     */
    public boolean desassociarTodos(CasoUso casoUso) {
        boolean success = true;
        LinkedList<Associado> remover = new LinkedList<Associado>();

        for (Associado a : associados) {
            if (a.getCasoUso().equals(casoUso)) {
                a.getRequisito().removerCasoUso(casoUso);
                casoUso.removerRequisito(a.getRequisito());
                a.marcarRemocao();
                entidadesAssociativasARemover.add(a);
                remover.add(a);
            }
        }

        for (Associado a : remover) {
            if (associados.remove(a) == false) {
                success = false;
            }
        }

        return success;
    }

    /**
     * 
     * @param requisito
     * @return
     */
    public boolean desassociarTodosTestes(Requisito requisito) {
        boolean success = true;
        LinkedList<Satisfazer> remover = new LinkedList<Satisfazer>();

        for (Satisfazer s : satisfacoes) {
            if (s.getRequisito().equals(requisito)) {
                s.getTeste().removerRequisito(requisito);
                requisito.removerTeste(s.getTeste());
                s.marcarRemocao();
                entidadesAssociativasARemover.add(s);
                remover.add(s);
            }
        }

        for (Satisfazer s : remover) {
            if (satisfacoes.remove(s) == false) {
                success = false;
            }
        }

        return success;
    }

    /**
     * 
     * @param requisito
     * @return
     */
    public boolean desassociarTodosCasosUso(Requisito requisito) {
        boolean success = true;
        LinkedList<Associado> remover = new LinkedList<Associado>();

        for (Associado a : associados) {
            if (a.getRequisito().equals(requisito)) {
                a.getCasoUso().removerRequisito(requisito);
                requisito.removerCasoUso(a.getCasoUso());
                a.marcarRemocao();
                entidadesAssociativasARemover.add(a);
                remover.add(a);
            }
        }

        for (Associado a : remover) {
            if (associados.remove(a) == false) {
                success = false;
            }
        }

        return success;
    }

    /**
     * 
     */
    private void preencherModelo() {
        LinkedList<Requisito> lRequisitos = new LinkedList<Requisito>();
        LinkedList<CasoUso> lCasos = new LinkedList<CasoUso>();
        LinkedList<Teste> lTestes = new LinkedList<Teste>();

        ResultSet rsGeral = null;
        ResultSet rsPontosExtensao = null;

        Projecto pTemp = null;
        Requisito rTemp = null;
        CasoUso cTemp = null;
        PontoExtensao eTemp = null;
        Teste tTemp = null;
        Logger.getLogger(Sistema.class.getName()).log(Level.INFO, "[SISTEMA]: Iniciar o modelo");
        try {
            Statement stmGestoes = Ligacao.getConnection().createStatement();
            ResultSet rsGestoes = stmGestoes.executeQuery("SELECT Projecto.id, Projecto.nome, Projecto.descricao FROM Projecto, Gerir WHERE idProjecto = Projecto.id AND idGestor = " + gestor.getId());

            PreparedStatement psRequisitos = Ligacao.getConnection().prepareStatement("SELECT * FROM Requisito WHERE idProjecto = ?");
            PreparedStatement psCasosUso = Ligacao.getConnection().prepareStatement("SELECT * FROM CasoUso WHERE idProjecto = ?");
            PreparedStatement psPontosExtensao = Ligacao.getConnection().prepareStatement("SELECT * FROM PontoExtensao WHERE idCasoUso = ?");
            PreparedStatement psTestes = Ligacao.getConnection().prepareStatement("SELECT * FROM Teste WHERE idProjecto = ?");
            PreparedStatement psAssociado = Ligacao.getConnection().prepareStatement("SELECT idCasoUso FROM Associado WHERE idRequisito = ?");
            PreparedStatement psSatisfazer = Ligacao.getConnection().prepareStatement("SELECT idTeste FROM Satisfazer WHERE idRequisito = ?");
            PreparedStatement psGestores = Ligacao.getConnection().prepareStatement("SELECT id FROM utilizador, gerir WHERE utilizador.id = gerir.idGestor AND gerir.idProjecto = ?");
            while (rsGestoes.next()) {
                pTemp = new Projecto(rsGestoes.getInt("Projecto.id"), rsGestoes.getString("Projecto.nome"), rsGestoes.getString("Projecto.descricao"), null, null, null, null, false, false);
                gestor.adicionarProjecto(pTemp);
                pTemp.adicionarGestor(gestor);
                gestores.add(new Gerir(pTemp, gestor, false));
                listaProjectos.add(pTemp);

                psGestores.setInt(1, pTemp.getId());
                rsGeral = psGestores.executeQuery();
                while (rsGeral.next()) {
                    int id = rsGeral.getInt("id");
                    for (Utilizador u : getUtilizadores()) {
                        if (u.getId() == id && u.getId() != gestor.getId()) {
                            pTemp.adicionarGestor((Gestor) u);
                            gestores.add(new Gerir(pTemp, (Gestor) u, false));
                        }
                    }
                }

                psRequisitos.setInt(1, pTemp.getId());
                rsGeral = psRequisitos.executeQuery();
                while (rsGeral.next()) {
                    rTemp = new Requisito(rsGeral.getInt("id"), rsGeral.getString("identificador"), rsGeral.getInt("origem"), rsGeral.getInt("risco"), rsGeral.getInt("prioridade"), rsGeral.getString("descricao"), null, null, pTemp, false, false);
                    pTemp.adicionarRequisito(rTemp);
                    rTemp.setProjecto(pTemp);
                    lRequisitos.add(rTemp);
                }

                psCasosUso.setInt(1, pTemp.getId());
                rsGeral = psCasosUso.executeQuery();
                while (rsGeral.next()) {
                    cTemp = new CasoUso(rsGeral.getInt("id"), rsGeral.getString("identificador"), rsGeral.getString("descricao"), null, pTemp, null, false, false);

                    psPontosExtensao.setInt(1, cTemp.getId());
                    rsPontosExtensao = psPontosExtensao.executeQuery();
                    while (rsPontosExtensao.next()) {
                        eTemp = new PontoExtensao(rsPontosExtensao.getInt("id"), rsPontosExtensao.getString("nome"), cTemp, false, false);
                        cTemp.adicionarPontoExtensao(eTemp);
                    }
                    pTemp.adicionarCasoUso(cTemp);
                    cTemp.setProjecto(pTemp);
                    lCasos.add(cTemp);
                }

                psTestes.setInt(1, pTemp.getId());
                rsGeral = psTestes.executeQuery();
                while (rsGeral.next()) {
                    tTemp = new Teste(rsGeral.getInt("id"), rsGeral.getString("nome"), tiposTeste.get(rsGeral.getInt("tipo")), rsGeral.getBoolean("realizado"), pTemp, null, false, false);
                    pTemp.adicionarTeste(tTemp);
                    tTemp.setProjecto(pTemp);
                    lTestes.add(tTemp);
                }

                ResultSet rsAssociado = null;
                ResultSet rsSatisfazer = null;

                int idCasoUso = 0, idTeste = 0;
                for (Requisito r : lRequisitos) {

                    psAssociado.setInt(1, r.getId());
                    rsAssociado = psAssociado.executeQuery();
                    while (rsAssociado.next()) {
                        idCasoUso = rsAssociado.getInt("idCasoUso");
                        for (CasoUso c : lCasos) {
                            if (c.getId() == idCasoUso) {
                                r.adicionarCasoUso(c);
                                c.adicionarRequisito(r);
                                associados.add(new Associado(c, r, false));
                                break;
                            }
                        }
                    }

                    psSatisfazer.setInt(1, r.getId());
                    rsSatisfazer = psSatisfazer.executeQuery();
                    while (rsSatisfazer.next()) {
                        idTeste = rsSatisfazer.getInt("idTeste");
                        for (Teste t : lTestes) {
                            if (t.getId() == idTeste) {
                                r.adicionarTeste(t);
                                t.adicionarRequisito(r);
                                satisfacoes.add(new Satisfazer(t, r, false));
                                break;
                            }
                        }
                    }

                }
            }
            psCasosUso.close();
            psRequisitos.close();
            psPontosExtensao.close();
            psTestes.close();
            psAssociado.close();
            psSatisfazer.close();
        } catch (SQLException ex) {
            Logger.getLogger(Sistema.class.getName()).log(Level.SEVERE, null, ex);
        }
        Logger.getLogger(Sistema.class.getName()).log(Level.INFO, "[SISTEMA]: Modelo carregado");
    }

    /**
     * 
     * @return
     */
    public LinkedList<TipoTeste> getTiposTeste() {
        LinkedList<TipoTeste> copia = new LinkedList<TipoTeste>();
        Collection<TipoTeste> tempCollection = tiposTeste.values();

        for (TipoTeste t : tempCollection) {
            copia.add(t);
        }

        return copia;
    }

    /**
     * 
     * @param aConfirmar
     * @return
     */
    public boolean usernameExiste(String aConfirmar) {
        if (listaUtilizadores.isEmpty()) {
            preencherListaUtilizadores();
        }
        for (Utilizador u : listaUtilizadores) {
            if (u.getUsername().equals(aConfirmar)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 
     * @return
     */
    private void preencherListaUtilizadores() {
        try {
            Utilizador temp;
            Statement stmt = Ligacao.getConnection().createStatement();
            ResultSet rs = stmt.executeQuery("SELECT id, nome, endereco, " +
                    "codPostal, codExtra, email,  msn, icq, sexo, " +
                    "dataNascimento, telefone, telemovel, u_username, " +
                    "u_password, tipo FROM Utilizador ORDER BY nome");

            while (rs.next()) {
                GregorianCalendar nascimento = null;
                Date tempDate;
                if ((tempDate = rs.getDate("dataNascimento")) != null) {
                    nascimento = new GregorianCalendar();
                    nascimento.setTime(tempDate);
                }
                if (rs.getString("tipo").equals("G")) {
                    temp = new Gestor(rs.getInt("id"), rs.getString("nome"), rs.getString("endereco"), rs.getString("codExtra"), codigosPostais.get(new Integer(rs.getInt("codPostal"))), rs.getString("email"), rs.getString("msn"), rs.getString("icq"), rs.getString("sexo"), nascimento, rs.getString("telefone"), rs.getString("telemovel"), rs.getString("u_username"), rs.getString("u_password"), null, false, false);
                } else if (rs.getString("tipo").equals("A")) {
                    temp = new Administrador(rs.getInt("id"), rs.getString("nome"), rs.getString("endereco"), rs.getString("codExtra"), codigosPostais.get(new Integer(rs.getInt("codPostal"))), rs.getString("email"), rs.getString("msn"), rs.getString("icq"), rs.getString("sexo"), nascimento, rs.getString("telefone"), rs.getString("telemovel"), rs.getString("u_username"), rs.getString("u_password"), false, false);
                } else {
                    temp = new Utilizador(rs.getInt("id"), rs.getString("nome"), rs.getString("endereco"), rs.getString("codExtra"), codigosPostais.get(new Integer(rs.getInt("codPostal"))), rs.getString("email"), rs.getString("msn"), rs.getString("icq"), rs.getString("sexo"), nascimento, rs.getString("telefone"), rs.getString("telemovel"), rs.getString("u_username"), rs.getString("u_password"), Utilizador.INDEFINIDO, false, false);
                }
                if (temp.getId() != autenticado.getId() && !utilizadorExisteNaLista(temp.getId())) {
                    temp.addListener(this);
                    listaUtilizadores.add(temp);
                }
            }
        } catch (SQLException ex) {
            Logger.getLogger(Sistema.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    
    /**
     * 
     * @param id
     * @return
     */
    private boolean utilizadorExisteNaLista(int id) {
        for(Utilizador u : listaUtilizadores) {
            if(u.getId() == id) {
                return true;
            }
        }
        return false;
    }

    /**
     * 
     * @return
     */
    public LinkedList<Utilizador> getUtilizadores() {
        LinkedList<Utilizador> lista = new LinkedList<Utilizador>();

        if (listaUtilizadores.isEmpty()) {
            preencherListaUtilizadores();
        }

        for (Utilizador u : listaUtilizadores) {
            lista.add(u);
        }

        return lista;
    }

    /**
     * 
     * @param caracteres
     * @return
     */
    public String gerarPalavraPasse(int caracteres) {
        StringBuffer strbff = new StringBuffer(caracteres * 3);
        char[] vogais = {'a', 'e', 'i', 'o', 'u'};
        char[] consoantes = {'b', 'c', 'd', 'g', 'h', 'j', 'k', 'l', 'm', 'n', 'p', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'};
        char[] numeros = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9'};


        for (int i = 0; i < caracteres; i++) {
            strbff.append(vogais[(int) (Math.random() * vogais.length)]);
            strbff.append(consoantes[(int) (Math.random() * consoantes.length)]);
            strbff.append(numeros[(int) (Math.random() * numeros.length)]);
        }

        int start = (int) (Math.random() * (2 * caracteres - 1));
        return strbff.substring(start, start + caracteres);
    }

    /**
     * 
     */
    public void gravar() {
        Logger.getLogger(Sistema.class.getName()).log(Level.INFO, "[SISTEMA]: Iniciar gravacao");
        if (gestor != null) {
            LinkedList<Projecto> lProjectos = gestor.getTodosProjectos();
            LinkedList<CasoUso> lCasoUso;
            LinkedList<Requisito> lRequisitos;
            LinkedList<Teste> lTestes;

            for (Gravavel g : entidadesAssociativasARemover) {
                g.gravar();
            }

            for (Gravavel g : tiposTesteAGravar) {
                g.gravar();
            }

            for (TipoTeste t : tiposTeste.values()) {
                t.gravar();
            }

            gestor.gravar();
            for (Gravavel g : utilizadoresAGravar) {
                g.gravar();
            }

            for (Projecto p : lProjectos) {
                p.gravar();
                lCasoUso = p.getListaTodosCasosUso();
                for (CasoUso u : lCasoUso) {
                    u.gravar();
                    for(PontoExtensao ponto : u.getTodosPontosExtensao()){
                        ponto.gravar();
                    }
                }

                lRequisitos = p.getListaTodosRequisitos();
                for (Requisito r : lRequisitos) {
                    r.gravar();
                }

                lTestes = p.getListaTodosTestes();
                for (Teste t : lTestes) {
                    t.gravar();
                }
            }

            for (Gravavel g : novasEntidadesAssociativas) {
                g.gravar();
            }

            entidadesAssociativasARemover.clear();
            novasEntidadesAssociativas.clear();
            utilizadoresAGravar.clear();
            tiposTesteAGravar.clear();
        } else {
            for (Gravavel g : utilizadoresAGravar) {
                g.gravar();
            }
            utilizadoresAGravar.clear();
        }
        Logger.getLogger(Sistema.class.getName()).log(Level.INFO, "[SISTEMA]: Gravacao terminada");
    }

    /**
     * 
     * @param novo
     */
    public void adicionarUtilizador(Utilizador novo) {
        listaUtilizadores.add(novo);
        utilizadoresAGravar.add(novo);
    }

    /**
     * 
     * @param removido
     */
    public void removerUtilizador(Utilizador removido) {
        removido.marcarRemocao();
        listaUtilizadores.remove(removido);
        if (!utilizadoresAGravar.contains(removido)) {
            utilizadoresAGravar.remove(removido);
        }
    }

    /**
     * 
     * @param p
     * @return
     */
    public boolean adicionarProjecto(Projecto p) {
        gestor.adicionarProjecto(p);
        p.adicionarGestor(gestor);
        Gerir g = new Gerir(p, gestor);
        novasEntidadesAssociativas.add(g);
        gestores.add(g);
        return listaProjectos.add(p);
    }

    /**
     * 
     * @param p
     * @return
     */
    public boolean remover(Projecto p) {
        LinkedList<Gerir> temp = new LinkedList<Gerir>();
        p.marcarRemocao();
        for (Gerir g : gestores) {
            if (g.getProjecto().equals(p)) {
                g.marcarRemocao();
                entidadesAssociativasARemover.add(g);
                temp.add(g);
            }
        }

        for (Gerir g : temp) {
            gestores.remove(g);
        }
        temp.clear();

        return listaProjectos.remove(p);
    }

    /**
     * 
     * @param casoUso
     * @return
     */
    public boolean remover(CasoUso casoUso) {
        casoUso.marcarRemocao();
        return true;
    }

    /**
     * 
     * @param requisito
     * @return
     */
    public boolean remover(Requisito requisito) {
        requisito.marcarRemocao();
        return true;
    }

    /**
     * 
     * @param teste
     * @return
     */
    public boolean remover(Teste teste) {
        teste.marcarRemocao();
        return true;
    }

    /**
     * 
     * @param codigo
     * @param localidade
     * @return
     */
    public CodigoPostal getCodigoPostal(String codigo, String localidade) {
        for (CodigoPostal c : codigosPostais.values()) {
            if (c.getCodigo().equals(codigo) && c.getLocalidade().equals(localidade)) {
                return c;
            }
        }
        return null;
    }

    /**
     * 
     * @return
     */
    public Utilizador getUtilizadorAutenticado() {
        return autenticado;
    }

    /**
     * 
     * @param novo
     */
    public void adicionarTipoTeste(TipoTeste novo) {
        if(!tipoExisteNaHash(novo.getId())) {
            tiposTeste.put(novo.getId(), novo);
            tiposTesteAGravar.add(novo);
        }
    }
    
    /**
     * 
     * @param id
     * @return
     */
    private boolean tipoExisteNaHash(int id) {
        Collection<TipoTeste> tempCollection = tiposTeste.values();

        for (TipoTeste t : tempCollection) {
            if(t.getId() == id) {
                return true;
            }
        }

        return false;
    }

    /**
     * 
     * @param tipo
     */
    public void removerTipoTeste(TipoTeste tipo) {
        tipo.marcarRemocao();
        tiposTeste.remove(tipo.getId());
        if (!tiposTesteAGravar.contains(tipo)) {
            tiposTesteAGravar.add(tipo);
        }
    }

    public boolean podeRemover(TipoTeste tipo) {
        try {
            PreparedStatement pstm = Ligacao.getConnection().prepareStatement("SELECT * FROM Teste WHERE tipo = ?");
            pstm.setInt(1, tipo.getId());
            
            ResultSet rs = pstm.executeQuery();
            return !rs.first();
        } catch (SQLException ex) {
            Logger.getLogger(Sistema.class.getName()).log(Level.SEVERE, null, ex);
        }
        return false;
    }
    //TODO:
    public void logout() {
        autenticado = gestor = null;
        gestor = null;
        codigosPostais.clear();
        tiposTeste.clear();
        listaUtilizadores.clear();
        satisfacoes.clear();
        gestores.clear();
        associados.clear();
        listaProjectos.clear();
        novasEntidadesAssociativas.clear();
        entidadesAssociativasARemover.clear();
        utilizadoresAGravar.clear();
        tiposTesteAGravar.clear();
    }

    /**
     * 
     * @param g
     */
    public void dataChanged(Gravavel g) {
        if (!utilizadoresAGravar.contains(g)) {
            utilizadoresAGravar.add(g);
        }
    }
}

