package requirementsmanager.model;

import requirementsmanager.model.listeners.GravavelListener;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;
import requirementsmanager.utils.Ligacao;

public class Teste implements Gravavel {
    private int id;
    private String nome;
    private TipoTeste tipo;
    private boolean realizado;
    private Projecto projecto;
    private LinkedList<Requisito> requisitos;
    
    
    private boolean novo;
    private boolean removido;    
    private LinkedList<GravavelListener> listeners;
    private boolean dirty;

    public Teste (int id, String nome, TipoTeste tipo, boolean realizado, Projecto projecto, LinkedList<Requisito> requisitos, boolean novo, boolean removido) {
        this.id = id;
        this.nome = nome;
        this.tipo = tipo;
        this.realizado = realizado;
        
        if(projecto == null) {
            throw new IllegalArgumentException("Projecto inválido!");
        }
        this.projecto = projecto;
        this.requisitos = (requisitos == null ? new LinkedList<Requisito>() : requisitos);
        
        this.novo = novo;
        this.removido = removido;
        dirty = false;
        listeners = new LinkedList<GravavelListener>();
    }
    
    public Teste (int id, String nome, TipoTeste tipo, boolean realizado, Projecto projecto, LinkedList<Requisito> requisitos) {
        this(id, nome, tipo, realizado, projecto, requisitos, true, false);
    }
    
    public int getId() {
        return id;
    }

    public String getNome () {
        return nome;
    }

    public void setNome (String nome) {
        this.nome = nome;
        dirty = true;
        fireDataChanged();
    }

    public TipoTeste getTipo () {
        return tipo;
    }

    public void setTipo (TipoTeste tipo) {
        if(tipo == null) {
            throw new IllegalArgumentException("Tipo de teste inválido");
        }
        this.tipo = tipo;
        dirty = true;
        fireDataChanged();
    }
    
    public boolean isRealizado () {
        return realizado;
    }

    public void setRealizado (boolean realizado) {
        this.realizado = realizado;
        dirty = true;
        fireDataChanged();
    }
    
    public Projecto getProjecto() {
        return projecto;
    }
    
    public void setProjecto(Projecto projecto) {
        if(projecto == null) {
            throw new IllegalArgumentException("Projecto inválido");           
        }
        this.projecto = projecto;
        dirty = true;
        fireDataChanged();
    }
    

    public boolean adicionarRequisito(Requisito requisito) {
        if(requisitos == null) {
            requisitos = new LinkedList<Requisito>();
        }
        requisitos.add(requisito);
        fireDataChanged();
        return true;
    }
    
    public boolean removerRequisito(Requisito requisito) {
        if(requisitos != null) {
            requisitos.remove(requisito);
            fireDataChanged();
            return true;
        }
        
        return false;
    }
    
    public LinkedList<Requisito> getRequisitos() {
        LinkedList<Requisito> copia = new LinkedList<Requisito>();
        if(requisitos != null) {
            for(Requisito r : requisitos) {
                if(!r.isRemoved()) {
                    copia.add(r);
                }
            }
        }
        return copia;
    }

    public boolean gravar() {
        Statement stm;
        ResultSet rs;
        boolean sucesso = false;

        if(novo && removido) {
            return true;
        }
        try {
            if (novo) {
                stm = Ligacao.getConnection().createStatement();
                String insert = "INSERT INTO Teste(nome, tipo, realizado, idProjecto) " +
                        "VALUES('" + nome + "', " + tipo.getId() + ", " + realizado + ", " + projecto.getId() + ")";
                
                sucesso = stm.executeUpdate(insert, Statement.RETURN_GENERATED_KEYS) == 1;

                if (sucesso) {
                    rs = stm.getGeneratedKeys();
                    if(rs.first()) {
                        id = rs.getInt(1);
                    }
                    novo = false;
                    dirty = false;
                }

            } else if (removido) {
                stm = Ligacao.getConnection().createStatement();
                String delete = "DELETE FROM Teste WHERE id = " + id;
                
                sucesso = stm.executeUpdate(delete) == 1;
                
                if(sucesso) {
                    removido = false;
                    dirty = false;
                }
            } else if (dirty) {
                stm = Ligacao.getConnection().createStatement();
                String update = "UPDATE Teste SET nome = '" + nome + "', tipo = " + tipo.getId() + 
                        ", realizado = " + realizado + ", idProjecto = " + projecto.getId() + " WHERE id = " + id;
                
                sucesso = stm.executeUpdate(update) == 1;
                
                if (sucesso) {
                    dirty = false;
                }
            }
        } catch (SQLException ex) {
            Logger.getLogger(Utilizador.class.getName()).log(Level.SEVERE, null, ex);
        }
        return sucesso;
    }

    @Override
    public String toString() {
        return nome;
    }

    public boolean isNew() {
        return novo;
    }

    public boolean isRemoved() {
        return removido;
    }

    public void fireDataChanged() {
        for(GravavelListener gl : listeners) {
            gl.dataChanged(this);
        }
    }

    public void addListener(GravavelListener g) {
        if(listeners == null) {
            listeners = new LinkedList<GravavelListener>();
        }
        
        listeners.add(g);
    }

    public void removeListener(GravavelListener g) {
        if(listeners != null) {
            listeners.remove(g);
        }
    }

    public void marcarRemocao() {
        removido = true;
        fireDataChanged();
    }
}

