package requirementsmanager.model;

import requirementsmanager.model.listeners.GravavelListener;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;
import requirementsmanager.utils.Ligacao;

public class TipoTeste implements Gravavel {

    private int id;
    private String nome;
    private String descricao;
    private boolean novo;
    private boolean removido;
    private LinkedList<GravavelListener> listeners;
    private boolean dirty;

    public TipoTeste(int id, String nome, String descricao, boolean novo, boolean removido) {
        this.nome = nome;
        this.descricao = descricao;
        this.id = id;

        this.novo = novo;
        this.removido = removido;
        dirty = false;

        listeners = new LinkedList<GravavelListener>();
        Logger.getLogger(Gestor.class.getName()).log(Level.INFO, "[TIPOTESTE]: \"" + nome + "\" <" + id + "> - Criar");
    }

    public TipoTeste(int id, String nome, String descricao) {
        this(id, nome, descricao, true, false);
    }

    public String getDescricao() {
        return descricao;
    }

    public void setDescricao(String descricao) {
        this.descricao = descricao;
        dirty = true;
        fireDataChanged();
    }

    public String getNome() {
        return nome;
    }

    public void setNome(String nome) {
        this.nome = nome;
        dirty = true;
        fireDataChanged();
    }

    public int getId() {
        return id;
    }

    @Override
    public String toString() {
        return nome;
    }

    public boolean gravar() {
        Statement stm;
        ResultSet rs;
        boolean sucesso = false;

        if(novo && removido) {
            return true;
        }
        try {
            if (novo) {
                stm = Ligacao.getConnection().createStatement();
                String insert = "INSERT INTO TipoTeste(nome, descricao) VALUES('" + nome +
                        "', '" + descricao + "')";

                sucesso = stm.executeUpdate(insert, Statement.RETURN_GENERATED_KEYS) == 1;

                if (sucesso) {
                    rs = stm.getGeneratedKeys();
                    if (rs.first()) {
                        id = rs.getInt(1);
                    }
                    novo = false;
                    dirty = false;
                }
            } else if (removido) {

                stm = Ligacao.getConnection().createStatement();
                String delete = "DELETE FROM TipoTeste WHERE id = " + id;
                
                sucesso = stm.executeUpdate(delete) == 1;

                if (sucesso) {
                    novo = false;
                    dirty = false;
                    removido = false;
                }
            } else if (dirty) {
                stm = Ligacao.getConnection().createStatement();
                String update = "UPDATE TipoTeste SET nome = '" + nome + "', '" +
                        descricao + "' WHERE id = " + id;

                sucesso = stm.executeUpdate(update) == 1;

                if (sucesso) {
                    dirty = false;
                }
            }
        } catch (SQLException ex) {
            Logger.getLogger(Utilizador.class.getName()).log(Level.SEVERE, null, ex);
        }
        return sucesso;
    }

    public boolean isNew() {
        return novo;
    }

    public boolean isRemoved() {
        return removido;
    }

    public void fireDataChanged() {
        for (GravavelListener gl : listeners) {
            gl.dataChanged(this);
        }
    }

    public void addListener(GravavelListener g) {
        if (listeners == null) {
            listeners = new LinkedList<GravavelListener>();
        }

        listeners.add(g);
    }

    public void removeListener(GravavelListener g) {
        if (listeners != null) {
            listeners.remove(g);
        }
    }

    public void marcarRemocao() {
        removido = true;
        fireDataChanged();
    }
}

