package requirementsmanager.model;

import com.twmacinta.util.MD5;
import java.io.UnsupportedEncodingException;
import requirementsmanager.model.listeners.GravavelListener;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.GregorianCalendar;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;
import requirementsmanager.utils.Ligacao;

public class Utilizador implements Gravavel {

    public static final int GESTOR = 0;
    public static final int ADMINISTRADOR = 1;
    public static final int INDEFINIDO = 3;
    public static final String SEXO_MASCULINO = "M";
    public static final String SEXO_FEMININO = "F";
    
    private static final MD5 ENCODER = new MD5();
    private int id;
    private String nome;
    private String endereco;
    private String codigoExtra;
    private CodigoPostal codigo;
    private String email;
    private String msn;
    private String icq;
    private String sexo;
    private GregorianCalendar dataNascimento;
    private String telefone;
    private String telemovel;
    private String username;
    private String password;
    private int tipo;
    private boolean novo;
    private boolean removido;
    private boolean dirty;
    private LinkedList<GravavelListener> listeners;

    public Utilizador(int id, String nome, String endereco, String codigoExtra, CodigoPostal codigo, String email, String msn, String icq, String sexo, GregorianCalendar dataNascimento, String telefone, String telemovel, String username, String password, int tipo, boolean novo, boolean removido) {
        this.nome = nome;
        this.endereco = endereco;
        this.codigoExtra = codigoExtra;
        this.codigo = codigo;
        this.email = email;
        this.msn = msn;
        this.icq = icq;
        this.sexo = sexo;
        this.dataNascimento = dataNascimento;
        this.telefone = telefone;
        this.telemovel = telemovel;
        this.username = username;
        if(novo) {
        this.password = md5EncodeString(password);
        } else {
            this.password = password;
        }
        this.tipo = tipo;
        this.id = id;

        this.novo = novo;
        this.removido = removido;
        dirty = false;

        listeners = new LinkedList<GravavelListener>();
    }

    public Utilizador(int id, String nome, String endereco, String codigoExtra, CodigoPostal codigo, String email, String msn, String icq, String sexo, GregorianCalendar dataNascimento, String telefone, String telemovel, String username, String password, int tipo) {
        this(id, nome, endereco, codigoExtra, codigo, email, msn, icq, sexo, dataNascimento, telefone, telemovel, username, password, tipo, true, false);
    }

    public Utilizador(String nome, int tipo) {
        this(-1, nome, "", "", null, "", "", "", "", null, "", "", "", "", tipo);
    }

    public GregorianCalendar getDataNascimento() {
        return dataNascimento;
    }
    
    public static String md5EncodeString(String str) {
        try {
            ENCODER.Init();
            ENCODER.Update(str, null);
            return ENCODER.asHex();
        } catch (UnsupportedEncodingException ex) {
            Logger.getLogger(Utilizador.class.getName()).log(Level.SEVERE, null, ex);
        }
        return str;
    }
 
    public void setDataNascimento(GregorianCalendar data) {
        this.dataNascimento = data;
        dirty = true;
        fireDataChanged();
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
        dirty = true;
        fireDataChanged();
    }

    public String getEndereco() {
        return endereco;
    }

    public void setEndereco(String endereco) {
        this.endereco = endereco;
        dirty = true;
        fireDataChanged();
    }

    public String getCodigoExtra() {
        return codigoExtra;
    }

    public void setCodigoExtra(String codigo) {
        codigoExtra = codigo;
        dirty = true;
        fireDataChanged();
    }

    public CodigoPostal getCodigoPostal() {
        return codigo;
    }

    public void setCodigoPostal(CodigoPostal codigo) {
        this.codigo = codigo;
        dirty = true;
        fireDataChanged();
    }

    public String getIcq() {
        return icq;
    }

    public void setIcq(String icq) {
        this.icq = icq;
        dirty = true;
        fireDataChanged();
    }

    public String getMsn() {
        return msn;
    }

    public void setMsn(String msn) {
        this.msn = msn;
        dirty = true;
        fireDataChanged();
    }

    public String getNome() {
        return nome;
    }

    public void setNome(String nome) {
        this.nome = nome;
        dirty = true;
        fireDataChanged();
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = md5EncodeString(password);
        dirty = true;
        fireDataChanged();
    }

    public String getSexo() {
        return sexo;
    }

    public void setSexo(String sexo) {
        this.sexo = sexo;
        dirty = true;
        fireDataChanged();
    }

    public String getTelefone() {
        return telefone;
    }

    public void setTelefone(String telefone) {
        this.telefone = telefone;
        dirty = true;
        fireDataChanged();
    }

    public String getTelemovel() {
        return telemovel;
    }

    public void setTelemovel(String telemovel) {
        this.telemovel = telemovel;
        dirty = true;
        fireDataChanged();
    }

    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = username;
        dirty = true;
        fireDataChanged();
    }

    public int getTipo() {
        return tipo;
    }
    
    public void setTipo(int tipo) {
        this.tipo = tipo;
        dirty = true;
        fireDataChanged();
    }

    public int getId() {
        return id;
    }

    public boolean isDirty() {
        return dirty;
    }
    
    public void setDirty(boolean dirty) {
        this.dirty = dirty;
        fireDataChanged();
    }

    @Override
    public String toString() {
        return nome;
    }

    public boolean gravar() {
        Statement stm;
        ResultSet rs;
        boolean sucesso = false;

        if(novo && removido) {
            return true;
        }
        try {
            if (novo) {
                stm = Ligacao.getConnection().createStatement();
                String insert = "INSERT INTO Utilizador(nome, endereco, codPostal, email, msn, icq, " +
                        "sexo, dataNascimento, telefone, telemovel, u_username, u_password, tipo) " +
                        "VALUES('" + nome + "', '" + endereco + "', " + (codigo != null ? codigo.getId() : "null") +
                        ", '" + email + "', '" + msn + "', '" + icq + "', '" + sexo + "', " +
                        (dataNascimento != null ? new java.sql.Date(dataNascimento.getTimeInMillis()) : "null") +
                        ", '" + telefone + "', '" + telemovel + "', '" + username + "', '" + password + "', '" +
                        (tipo == Utilizador.GESTOR ? "G" : (tipo == Utilizador.ADMINISTRADOR) ? "A" : "I") + "')";

                sucesso = stm.executeUpdate(insert, Statement.RETURN_GENERATED_KEYS) == 1;

                if (sucesso) {
                    rs = stm.getGeneratedKeys();
                    if (rs.first()) {
                        id = rs.getInt(1);
                    }
                    novo = false;
                    dirty = false;
                    Logger.getLogger(Utilizador.class.getName()).log(Level.INFO, "[UTILIZADOR]: " + id + " - Inserido");
                }
            } else if (removido) {
                String delete = "DELETE FROM Utilizador WHERE id = " + id;
                stm = Ligacao.getConnection().createStatement();
                sucesso = stm.executeUpdate(delete) == 1;
                if (sucesso) {
                    novo = false;
                    dirty = false;
                    removido = false;
                    Logger.getLogger(Utilizador.class.getName()).log(Level.INFO, "[UTILIZADOR]: " + id + " - Removido");
                }
            } else if (dirty) {
                stm = Ligacao.getConnection().createStatement();
                String update = "UPDATE Utilizador SET nome = '" + nome + "', endereco = '" +
                        endereco + "', codPostal = " + (codigo != null ? codigo.getId() : "null") +
                        ", email = '" + email + "', msn = '" + msn + "', icq = '" + icq + "', sexo = '" + sexo +
                        "', dataNascimento = " + (dataNascimento != null ? new java.sql.Date(dataNascimento.getTimeInMillis()) : "null") +
                        ", telefone = '" + telefone + "', telemovel = '" + telemovel + "', u_username = '" +
                        username + "', u_password = '" + password + "', tipo = '" + (tipo == Utilizador.GESTOR ? "G" : "A") +
                        "' WHERE id = " + id;

                sucesso = stm.executeUpdate(update) == 1;

                if (sucesso) {
                    dirty = false;
                    Logger.getLogger(Utilizador.class.getName()).log(Level.INFO, "[UTILIZADOR]: " + id + " - Actualizado");
                }
            }
        } catch (SQLException ex) {
            Logger.getLogger(Utilizador.class.getName()).log(Level.SEVERE, null, ex);
        }
        return sucesso;
    }

    public boolean isNew() {
        return novo;
    }

    public boolean isRemoved() {
        return removido;
    }

    public void fireDataChanged() {
        for (GravavelListener gl : listeners) {
            gl.dataChanged(this);
        }
    }

    public void addListener(GravavelListener g) {
        if (listeners == null) {
            listeners = new LinkedList<GravavelListener>();
        }

        listeners.add(g);
    }

    public void removeListener(GravavelListener g) {
        if (listeners != null) {
            listeners.remove(g);
        }
    }

    public void marcarRemocao() {
        removido = true;
        novo = false;
        dirty = false;
        fireDataChanged();
    }
}

