DROP TABLE IF EXISTS PontoExtensao;
DROP TABLE IF EXISTS Associado;
DROP TABLE IF EXISTS CasoUso;
DROP TABLE IF EXISTS Satisfazer;
DROP TABLE IF EXISTS Requisito;
DROP TABLE IF EXISTS Teste;
DROP TABLE IF EXISTS Gerir;
DROP TABLE IF EXISTS Projecto;
DROP TABLE IF EXISTS Utilizador;
DROP TABLE IF EXISTS TipoTeste;
DROP TABLE IF EXISTS CodigoPostal;

CREATE TABLE CodigoPostal(
	id INTEGER NOT NULL AUTO_INCREMENT,
	codigo VARCHAR(4) NOT NULL,
    localidade VARCHAR(50) NOT NULL,
	CONSTRAINT PK_codigoPostal PRIMARY KEY (id)
)ENGINE=InnoDB;

CREATE TABLE TipoTeste(
    id INTEGER NOT NULL AUTO_INCREMENT,
	nome VARCHAR(50),
    descricao VARCHAR(250),
	CONSTRAINT PK_tipoTeste PRIMARY KEY (id)
)ENGINE=InnoDB;

CREATE TABLE Utilizador(
	id INTEGER NOT NULL AUTO_INCREMENT,
	nome VARCHAR(150) NOT NULL,
    endereco VARCHAR(150),
	codPostal INTEGER,
	codExtra VARCHAR(3),
    email VARCHAR(150),
    msn VARCHAR(150),
    icq VARCHAR(150),
    sexo VARCHAR(1) NOT NULL,
    dataNascimento DATE,
    telefone VARCHAR(12),
    telemovel VARCHAR(12),
    u_username VARCHAR(20) NOT NULL UNIQUE,
    u_password VARCHAR(32) NOT NULL,
	tipo VARCHAR(1) NOT NULL,
	CONSTRAINT PK_utilizador PRIMARY KEY (id),
	CONSTRAINT FK_codPostal_utilizador FOREIGN KEY (codPostal) REFERENCES CodigoPostal(id),
	CONSTRAINT CK_sexo_utilizador CHECK (sexo IN('M', 'F')),
	CONSTRAINT CK_tipo_utilizador CHECK (tipo IN('A', 'G', "I"))
)ENGINE=InnoDB;

CREATE TABLE Projecto(
	id INTEGER NOT NULL AUTO_INCREMENT,
    nome VARCHAR(100) NOT NULL,
	descricao VARCHAR(250),
	CONSTRAINT PK_projecto PRIMARY KEY (id)
)ENGINE=InnoDB;

/*Relaciona projectos com utilizadores*/
CREATE TABLE Gerir(
	idGestor INTEGER NOT NULL,
	idProjecto INTEGER NOT NULL,
	CONSTRAINT PK_gerir PRIMARY KEY (idGestor, idProjecto),
	CONSTRAINT FK_idProjecto_gerir FOREIGN KEY (idGestor) REFERENCES Utilizador(id),	
	CONSTRAINT FK_idGestor_gerir FOREIGN KEY (idProjecto) REFERENCES Projecto(id)
)ENGINE=InnoDB;

CREATE TABLE Teste(
	id INTEGER NOT NULL AUTO_INCREMENT,
	nome VARCHAR(100) NOT NULL, 
    tipo INTEGER NOT NULL,
	realizado TINYINT(1),
	idProjecto INTEGER NOT NULL,
	CONSTRAINT PK_teste PRIMARY KEY (id),
	CONSTRAINT FK_idProjecto_teste FOREIGN KEY (idProjecto) REFERENCES Projecto(id),
	CONSTRAINT FK_tipo_teste FOREIGN KEY (tipo) REFERENCES TipoTeste(id),
	CONSTRAINT CK_realizado_tipoTeste CHECK(realizado IN (0, 1))
)ENGINE=InnoDB;
	
CREATE TABLE Requisito(
	id INTEGER NOT NULL AUTO_INCREMENT,
	identificador VARCHAR(30) NOT NULL,
	origem TINYINT(1) NOT NULL,
	risco TINYINT(1) NOT NULL,
	prioridade TINYINT(1) NOT NULL,
	descricao VARCHAR(250),
	idProjecto INTEGER NOT NULL,
	CONSTRAINT PK_requisito PRIMARY KEY (id),
	CONSTRAINT FK_idProjecto_requisito FOREIGN KEY (idProjecto) REFERENCES Projecto(id),
	CONSTRAINT CK_origem CHECK (origem IN (1, 2)),
	CONSTRAINT CK_risco CHECK (risco IN(1, 2, 3)),
	CONSTRAINT CK_prioridade CHECK (prioridade IN(1, 2, 3))
)ENGINE=InnoDB;

/*Relaciona testes com os requisitos que os mesmos satisfazem*/
CREATE TABLE Satisfazer(
	idTeste INTEGER NOT NULL,
	idRequisito INTEGER NOT NULL,
	CONSTRAINT PK_satisfazer PRIMARY KEY (idTeste, idRequisito),
	CONSTRAINT FK_idTeste_satisfazer FOREIGN KEY (idTeste) REFERENCES Teste(id),
	CONSTRAINT FK_idRequisito_satisfazer FOREIGN KEY (idRequisito) REFERENCES Requisito(id)
)ENGINE=InnoDB;
	
CREATE TABLE CasoUso(
	id INTEGER NOT NULL AUTO_INCREMENT,
	identificador VARCHAR(30),
    descricao VARCHAR(250),
	idProjecto INTEGER NOT NULL,
	CONSTRAINT PK_casoUso PRIMARY KEY (id),
	CONSTRAINT FK_idProjecto_casoUso FOREIGN KEY (idProjecto) REFERENCES Projecto(id)
)ENGINE=InnoDB;

/*Relaciona requisitos com os casos de uso a que deram origem*/
CREATE TABLE Associado(
	idCasoUso INTEGER NOT NULL,
	idRequisito INTEGER NOT NULL,
	CONSTRAINT PK_associado PRIMARY KEY (idCasoUso, idRequisito),
	CONSTRAINT FK_idCasoUso_associado FOREIGN KEY (idCasoUso) REFERENCES CasoUso(id),
	CONSTRAINT FK_idRequisito_associado FOREIGN KEY (idRequisito) REFERENCES Requisito(id)
)ENGINE=InnoDB;

/*Entidade fraca que representa os pontos de extensão associados a um caso de uso*/
CREATE TABLE PontoExtensao(
	id INTEGER NOT NULL AUTO_INCREMENT,
	idCasoUso INTEGER NOT NULL,
	nome VARCHAR(250) NOT NULL,
	CONSTRAINT PK_pontoExtensao PRIMARY KEY (id, idCasoUso),
	CONSTRAINT FK_idCasoUso_pontoExtensao FOREIGN KEY (idCasoUso) REFERENCES CasoUso(id)
)ENGINE=InnoDB;

/*Valores básicos suportados pela aplicação*/
INSERT INTO TipoTeste(nome, descricao) VALUES("Unidade", "");
INSERT INTO TipoTeste(nome, descricao) VALUES("Funcionalidade", "");
INSERT INTO TipoTeste(nome, descricao) VALUES("Usabilidade", "");